/* ***************************************************************** *
 * Send Mail                                                         *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.1.0 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Sat 16 Jan 1999 *
 * Modification date . . . . . . . . . . . . . . . . Sun 04 Apr 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standard includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <unistd.h>


/* ***************************************************************** *
 * network includes                                                  *
 * ***************************************************************** */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


/* ***************************************************************** *
 * custom includes                                                   *
 * ***************************************************************** */
#include "mail.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    1024


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static const char   program[]               = "Mail";
static const char   version[]               = "1.1.0";
static const char   crlf[]                  = "\015\12";



/* ***************************************************************** *
 * Mail::Mail()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This is the Mail constructor.                                   *
 *                                                                   *
 * ***************************************************************** */
Mail::Mail(char * sender, char * recipient, char * subject, char * message) {

   printf("AVANT\n");
    if(sender != NULL && strlen(sender) > 0) {
        _sender = new char[strlen(sender)+1];
        strcpy(_sender, sender);
    } else {
        _sender = NULL;
    }
    
    if(recipient != NULL && strlen(recipient) > 0) {
        _recipient = new char[strlen(recipient)+1];
        strcpy(_recipient, recipient);
    } else {
        _recipient = NULL;
    }
    
    if(subject != NULL && strlen(subject) > 0) {
        _subject = new char[strlen(subject)+1];
        strcpy(_subject, subject);
    } else {
        _subject = NULL;
    }
    
    if(message != NULL && strlen(message) > 0) {
        _message = new char[strlen(message)+1];
        strcpy(_message, message);
    } else {
        _message = NULL;
    }
    printf("APRES\n");
}


/* ***************************************************************** *
 * Mail::~Mail()                                                     *
 * -----------                                                       *
 * description                                                       *
 *   This is the Mail destructor.                                    *
 *                                                                   *
 * ***************************************************************** */
Mail::~Mail(void) {
    if(_recipient != NULL)  delete [] _recipient;
    if(_subject   != NULL)  delete [] _subject;
    if(_message   != NULL)  delete [] _message;
}


/* ***************************************************************** *
 * Mail::Server()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the server field access method.                         *
 *   If the argument is NULL, the server field is constructed using  *
 *   the recipient field (we take the string following the '@', as   *
 *   it should be).                                                  *
 *                                                                   *
 * ***************************************************************** */
void Mail::Server(char * server) {
    if(server == NULL) {
        char * s = strchr(_recipient, '@');
        if(s == NULL) throw Mail_MalformedAddress();
        s++;
        _server = new char[strlen(s)];
        strcpy(_server, s);
        return;
    }
    
    if(server != NULL && strlen(server) > 0) {
        if(_server != NULL) {
            delete [] _server;
        }
        _server = new char[strlen(server)];
        strcpy(_server, server);
    }
}


/* ***************************************************************** *
 * Mail::Sender()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the sender field access method.                         *
 *                                                                   *
 * ***************************************************************** */
void Mail::Sender(char * sender) {
    if(sender != NULL && strlen(sender) > 0) {
        if(_sender != NULL) {
            delete [] _sender;
        }
        _sender = new char[strlen(sender)];
        strcpy(_sender, sender);
    }
}


/* ***************************************************************** *
 * Mail::Recipient()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This is the recipient field access method.                      *
 *                                                                   *
 * ***************************************************************** */
void Mail::Recipient(char * recipient) {
    if(recipient != NULL && strlen(recipient) > 0) {
        if(_recipient != NULL) {
            delete [] _recipient;
        }
        _recipient = new char[strlen(recipient)];
        strcpy(_recipient, recipient);
    }
}


/* ***************************************************************** *
 * Mail::Subject()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the subject field access method.                        *
 *                                                                   *
 * ***************************************************************** */
void Mail::Subject(char * subject) {
    if(subject != NULL && strlen(subject) > 0) {
        if(_subject != NULL) {
            delete [] _subject;
        }
        _subject = new char[strlen(subject)];
        strcpy(_subject, subject);
    }
}


/* ***************************************************************** *
 * Mail::Message()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the message field access method.                        *
 *                                                                   *
 * ***************************************************************** */
void Mail::Message(char * message) {
    if(message != NULL && strlen(message) > 0) {
        if(_message != NULL) {
            delete [] _message;
        }
        _message = new char[strlen(message)];
        strcpy(_message, message);
    }
}


/* ***************************************************************** *
 * Mail::Send()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function sends the mail.                                   *
 *                                                                   *
 * exceptions                                                        *
 *   Mail_NoRecipient                                                *
 *       No Recipient was given.                                     *
 *                                                                   *
 *   Mail_NoMessage                                                  *
 *       No message was given.                                       *
 *                                                                   *
 * ***************************************************************** */
void Mail::Send(void) {
    time_t  now = time(0);
    
    if(_recipient == NULL || strlen(_recipient) <= 1)  throw Mail_NoRecipient();
    if(_message   == NULL || strlen(_message  ) <= 1)  throw Mail_NoMessage();

    
   FILE * pipo = fopen("/tmp/pipo.mail","w");
   if ( ! pipo ) throw Mail_NoMessage();
    printf("YO\n");

        fprintf(pipo,"Subject: %s\n", _subject);
    printf("YO\n");
        fprintf(pipo,"Date: %s\n", ctime(&now));
    printf("YO\n");
        fprintf(pipo,"\n");
    printf("YO\n");
        fprintf(pipo,"%s",_message);
    printf("YO\n");
        fprintf(pipo,".");
    printf("YO\n");
	fclose(pipo);
    printf("YO\n");

	char buffer[400];
	sprintf(buffer,"cat /tmp/pipo.mail | /usr/sbin/sendmail -F %s %s", _sender, _recipient);

	system (buffer);
}

