/*********************************************************************
 *                                 *  version          date          *
 *  bot_rep.c                      *    0.6.1            15/04/1998  *
 *                                 *                                 *
 *********************************************************************
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation; either version 2, or   *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to the Free Software       *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ***************************************************************** *
 * This Program has been made by Sebastien Aperghis-Tramoni from     *
 * The Madness Brothers. If you find any bug, or if you do any       *
 * improvment, please contact send him a mail.                       *
 *                                                                   *
 *   Mail: Sebastien.Aperghis@es2i.esil.univ-mrs.fr                  *
 *     or  madingue@tango.resus.univ-mrs.fr                          *
 *   Web:  http://www.resus.org/~madingue                            *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>


/* ***************************************************************** *
 * custom includes                                                   *
 * ***************************************************************** */
#include "bot_rep.H"
#include "G_Path.H"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */


/* ***************************************************************** *
 * salt                                                              *
 * ***************************************************************** */
#define and		&&
#define or		||
#define not		!
#define elsif	else if


/* ***************************************************************** *
 * types                                                             *
 * ***************************************************************** */
typedef struct condition {
	char *              word;
	char                op;
	struct condition *  next;
} condition_t;

typedef struct condition *  CONDITION;

typedef struct bloc {
	struct condition *  condition;
	int                 nb_phrases;
	char **             phrases;
	
	int                 kick_at;
	int                 nb_kick_phrases;
	char **             kick_phrases;
} bloc_t;

typedef struct bloc *       BLOC;


/* ***************************************************************** *
 * prototypes                                                        *
 * ***************************************************************** */
int         next_char           (void);
int         next_useful_char    (void);
int         strip_comments      (void);

int         bloc                (BLOC *);
int         condition           (CONDITION *);
int         kick_at             (int *);
int         phrase              (char **);
int         word                (char **);
int         number              (int *);

CONDITION   new_condition       (char *, char, CONDITION);
BLOC        new_bloc            (CONDITION, int, char **);
void        delete_condition    (CONDITION);

int         dictionary_parse    (BLOC, char *, char **);
int         condition_test      (char *, CONDITION);
int         condition_count     (CONDITION);
int         lower_case          (char *);
int         irand               (int);
int         warn                (char *, int, char *);
int         die                 (char *, int, char *);


/* ***************************************************************** *
 * macros                                                            *
 * ***************************************************************** */
#define digit(C)            ( (C >= '0') && (C <= '9') )
#define letter(C)           ( (C > 32) && (C < 127) && (C != 41) )
#define operator(C)         ( (C == AND) || (C == OR) )
#define space(C)            ( C <= 32 )
#define value(C)            ( C - '0' )

#define index(STR,SUBSTR)	( !!strstr(STR,SUBSTR) )
#define length(STR)			( strlen(STR) )
#define eq(STR1,STR2)		( strcmp(STR1,STR2) == 0 )

#define type_on_keyboard(S) ( (TYPE_DELAY) ? sleep((int)(strlen(S) * TYPE_RATE)) : 0 )


/* ***************************************************************** *
 * extern variables                                                  *
 * ***************************************************************** */
extern G_Path      G            ;


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char        wchar        ;
static FILE *      file         ;
static BLOC *      corpus       ;
static int         corpus_items ;
static BLOC        French_Dict  ;
static BLOC        Spanish_Dict ;
static BLOC        German_Dict  ;
static BLOC        Dutch_Dict   ;
static BLOC        Italian_Dict ;
static const char  AND          = '&';
static const char  OR           = '|';
static long int    used_mem     = 0;
static long int    globals_mem  = 0;
static long int    const_mem    = 0;
static float       TYPE_RATE    = 0.1;
static short int   TYPE_DELAY   = 0;


/* ***************************************************************** *
 * constants                                                         *
 * ***************************************************************** */
const int       BOTHERED        = 8;

const int       MAX_LOG_LENGTH  = 500;
const int       MAX_ANS_LENGTH  = 500;
const int       MAX_BUF         = 500;
const int       CORPUS_SIZE     = 200;

const char *    DefaultLogStr   = "mm.. an error probably occured...";
const char *    Confirm         = "__YES__";

const char *    ActivateTypeDelayCommand    = "__ACTIVATE_TYPE_DELAY__";
const char *    AuthorDisplayCommand        = "__AUTHOR__";
const char *    CommandsListingCommand      = "__COMMANDS__";
const char *    DataStatisticsCommands      = "__DATA__";
const char *    DateOfCompilationCommand    = "__DATE__";
const char *    DesactivateTypeDelayCommand = "__DESACTIVATE_TYPE_DELAY__";
const char *    HelpDisplayCommand          = "__HELP__";
const char *    KickCommand                 = "__KICK__";
const char *    ReloadCommand               = "__RELOAD__";
const char *    SizeDisplayCommand          = "__SIZE__";
const char *    TestLowerCaseCommand        = "__TEST_LOWER_CASE__";
const char *    TestRandomGenerationCommand = "__TEST_RANDOM__";
const char *    TimeCommand                 = "__TIME__";
const char *    TypeDelayCommand            = "__TYPE_DELAY__";
const char *    VersionDisplayCommand       = "__VERSION__";


/*
 * CORPUS_SIZE is the maximum number of item in the corpus
 * MAX_BUF is the maximum length of a sentence
 * MAX_LOG_LENGTH is the maximum length of the log string
 * MAX_ANS_LENGTH is the maximum length of the answer
 */

#define kXMSG                   'X'
#define kEMOT                   's'
#define kDRAW                   'd'
#define kKICK                   'k'
#define kSYSG                   'h'
#define kSYSO                   'S'

#define TOTAL                   0
#define X_MSG                   1
#define EMOTE                   2
#define DRAWING                 3
#define LAST_TYPE               3

#define DEFAULT                 0
#define SPANISH                 1
#define FRENCH                  2
#define WHO_IS_THE_BOT          3
#define RESERVED                4
#define GOD                     5
#define LIKED                   6
#define DISLIKED                7
#define RESUS                   8
#define INSULTED                9
#define BUG                    10
#define HOW_IS_THE_BOT         11
#define LAST_MSG_TYPE          11



/* ***************************************************************** *
 * =========================== FUNCTIONS =========================== *
 * ***************************************************************** */



/* ***************************************************************** *
 * bot_rep                                                           *
 * -------                                                           *
 *   description                                                     *
 *     This is the bot constructor.                                  *
 *                                                                   *
 * ***************************************************************** */
bot_rep::bot_rep(void) {
	bot_rep::init_bot();
}


/* ***************************************************************** *
 * ~bot_rep                                                          *
 * --------                                                          *
 *   description                                                     *
 *     This is the bot destructor.                                   *
 *                                                                   *
 * ***************************************************************** */
bot_rep::~bot_rep(void) {
	bot_rep::kill_bot();
}


/* ***************************************************************** *
 * bot_reponse                                                       *
 * -----------                                                       *
 *   description                                                     *
 *     This is the main function of the bot.                         *
 *                                                                   *
 * ***************************************************************** */
char * bot_rep::bot_reponse(char * message, int user, char msg_type, char ** answer, char * ans_type) {
	char *	log_str = NULL;
	int		i = 0;
	int 	r = 0;
	
	// Allocating memory for the return string
	log_str = new char [MAX_LOG_LENGTH];
	if(not log_str) {
		warn("bot_reponse", __LINE__, "'new char []' failed");
		return NULL;
	}
	
	// Allocating memoru for the answer string. 
	// If it is not used by the commands, it will 
	// be free()'d below. 
	*answer = new char[MAX_ANS_LENGTH];
	if(not *answer) {
		warn("bot_reponse", __LINE__, "'new char []' failed");
		return NULL;
	}
	
	
	// Help Display Command
	if(index(message, HelpDisplayCommand)) {
		
		sprintf(log_str, "user #%d asked the bot for help\n", user);
		
		// Help: Activate Type Delay Command
		if(index(message, ActivateTypeDelayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command activate the type delay features.\n  Needs confirm.", ActivateTypeDelayCommand);
			return log_str;
		}
		
		// Help: Author Display Command
		if(index(message, AuthorDisplayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays informations about the author.", AuthorDisplayCommand);
			return log_str;
		}
		
		// Help: Commands Listing Command
		if(index(message, CommandsListingCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the available commands.", CommandsListingCommand);
			return log_str;
		}
		
		// Help: Data Statistics Commands
		if(index(message, DataStatisticsCommands)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays statistics about the dictionaries.", DataStatisticsCommands);
			return log_str;
		}
		
		// Help: Date Of Compilation Command
		if(index(message, DateOfCompilationCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the date and time of compilation.", DateOfCompilationCommand);
			return log_str;
		}
		
		// Help: Desactivate Type Delay Command
		if(index(message, DesactivateTypeDelayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command desctivate the type delay feature.", DesactivateTypeDelayCommand);
			return log_str;
		}
		
		// Help: Help Display Command
		if(index(message, HelpDisplayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command display help about other commands.", HelpDisplayCommand);
			return log_str;
		}
		
		// Help: Kick Command
		if(index(message, KickCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command allow you to kick someone.\n  Need confirm.", KickCommand);
			return log_str;
		}
		
		// Help: Reload Command
		if(index(message, ReloadCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command reloads the dictionaries.\n  Need confirm.", ReloadCommand);
			return log_str;
		}
		
		// Help: Size Display Command
		if(index(message, SizeDisplayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the size used by the data of the bot.", SizeDisplayCommand);
			return log_str;
		}
		
		// Help: Test Lower Case Command
		if(index(message, TestLowerCaseCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command passes in lower case the message.", TestLowerCaseCommand);
			return log_str;
		}
		
		// Help: Test Random Generation Command
		if(index(message, TestRandomGenerationCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays 10 results of the irand() function.", TestRandomGenerationCommand);
			return log_str;
		}
		
		// Help: Time Command
		if(index(message, TimeCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the elapsed time since the boot.", TimeCommand);
			return log_str;
		}
		
		// Help: Type Delay Command
		if(index(message, TypeDelayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the current state of the type delay feature.", TypeDelayCommand);
			return log_str;
		}
		
		// Help: Version Display Command
		if(index(message, VersionDisplayCommand)) {
			sprintf(*answer, "\\w%s\\c\n  This command displays the current version of the bot.", VersionDisplayCommand);
			return log_str;
		}
		
		sprintf(*answer, "\\wusage:\\c\n  __HELP__  <command>.");
		return log_str;
	}
	
	// Activate Type Delay Command
	if(index(message, ActivateTypeDelayCommand)) {
		
		if(index(message, Confirm)) {
			TYPE_DELAY = 1;
			sprintf(*answer, "\\wtype delay \\gactivated\\w.\ntype rate = \\y%1.2f \\csec", TYPE_RATE);
			sprintf(log_str, "user #%d activated type delay\n", user);
			return log_str;
			
		} else {
			sprintf(*answer, "You can't do that. ");
			sprintf(log_str, "user #%d try to activate the type delay, but didn't confirm\n", user);
			return log_str;
		}
	}
	
	// Author Display Command
	if(index(message, AuthorDisplayCommand)) {
		sprintf(*answer, "Guess who.. \\w:=)");
		sprintf(log_str, "user #%d asked who's the author\n", user);
		return log_str;
	}
	
	// Commands Listing Command
	if(index(message, CommandsListingCommand)) {
		sprintf(*answer, "Current available commands are : %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s", 
			ActivateTypeDelayCommand, 
			AuthorDisplayCommand, 
			CommandsListingCommand, 
			DataStatisticsCommands, 
			DateOfCompilationCommand, 
			DesactivateTypeDelayCommand, 
			HelpDisplayCommand, 
			ReloadCommand, 
			SizeDisplayCommand, 
			TestLowerCaseCommand, 
			TestRandomGenerationCommand, 
			TimeCommand, 
			TypeDelayCommand, 
			VersionDisplayCommand
			);
		sprintf(log_str, "user #%d asked to know the commands of the bot\n", user);
		return log_str;
	}
	
	// Data Statistics Commands
	if(index(message, DataStatisticsCommands)) {
		sprintf(*answer, "\\gData statistics\n\\w---------------\n\\w|\\c corpus  : \\y%d \\cdeclarations\n\\w| - - - - - - -\n\\w|\\c French  : \\y%d \\cwords\n\\w|\\c Spanish : \\y%d \\cwords\n\\w|\\c German  : \\y%d \\cwords\n\\w|\\c Dutch   : \\y%d \\cwords\n\\w|\\c Italian : \\y%d \\cwords", 
			corpus_items, 
			condition_count(French_Dict->condition), 
			condition_count(Spanish_Dict->condition), 
			condition_count(German_Dict->condition), 
			condition_count(Dutch_Dict->condition), 
			condition_count(Italian_Dict->condition)
			);
		sprintf(log_str, "user #%d asked for the data used by the bot\n", user);
		return log_str;
	}
	
	// Date Of Compilation Commands
	if(index(message, DateOfCompilationCommand)) {
		sprintf(*answer, "%s, %s", __DATE__, __TIME__);
		sprintf(log_str, "user #%d asked for the compilation date of the bot\n", user);
		return log_str;
	}
	
	// Desactivate Type Delay Command
	if(index(message, DesactivateTypeDelayCommand)) {
		TYPE_DELAY = 0;
		sprintf(*answer, "\\wtype delay \\rdesactivated\\w.");
		sprintf(log_str, "user #%d desactivated type delay\n", user);
		return log_str;
	}
	
	// Kick Command
	if(index(message, KickCommand)) {
		
		if(0) { // *** not implemented *** //
			sprintf(*answer, "\\rnot implemented \\w:=pp");
			sprintf(log_str, "user #%d tried something\n", user);
			return log_str;
		}
		
		if(index(message, Confirm)) {
			
			i = 0;
			r = 0;
			// search for the first digit
			while((message[i] != 0) and not digit(message[i])) {
				i++;
			}
			// read the number
			while((message[i] != 0) and digit(message[i])) {
				r = r * 10 + value(message[i]);
				i++;
			}
			
			if((r == 3) or (r == 77)) {
				// Vdb (3)
				// Maddingue (77)
				// hehe :=)
				sprintf(*answer, "\\cil aime \\rpas\\c ce genre de gag \\w:=pp");
				*ans_type = kKICK;
				sprintf(log_str, "user #%d tried to kick user#%d\n", user, r);
				return log_str;
			
			} else {
				sprintf(*answer, "\\rnot implemented \\w:=pp");
				// appel a kick()
				sprintf(log_str, "user #%d kicked user#%d\n", user, r);
				return log_str;
			
			}
			
		} else {
			sprintf(*answer, "You can't do that. ");
			sprintf(log_str, "user #%d try to kick someone, but didn't confirm\n", user);
			return log_str;
		}
	}
	
	// ReloadCommand
	if(index(message, ReloadCommand)) {
		
		if(index(message, Confirm)) {
			used_mem = 0;
			i = bot_rep::kill_bot();
			r = bot_rep::init_bot();
			
			sprintf(*answer, "Re-initialization completed.\n \\wkill_bot()\\c returned \\y%d\n \\winit_bot()\\c returned \\y%d", i, r);
			sprintf(log_str, "user #%d re-initialized the bot\n", user);
			return log_str;
			
		} else {
			sprintf(*answer, "You can't do that. ");
			sprintf(log_str, "user #%d try to re-initialized the bot, but didn't confirm\n", user);
			return log_str;
		}
	}
	
	// Size Display Command
	if(index(message, SizeDisplayCommand)) {
		sprintf(*answer, "\\gMemory use estimation\n\\w---------------------\n\\w|\\c globals : \\y%ld \\cbytes\n\\w|\\c   const : \\y%ld \\cbytes\n\\w|\\c    data : \\y%ld \\ybytes\n\\w| - - - - - - - - - -\n\\w|\\c   total : \\y%ld \\cbytes", globals_mem, const_mem, used_mem, globals_mem + const_mem + used_mem);
		sprintf(log_str, "user #%d asked for the size used by the bot\n", user);
		return log_str;
	}
	
	// Test Lower Case Command
	if(index(message, TestLowerCaseCommand)) {
		i = lower_case(message);
		sprintf(*answer, "%s\n\\clength: \\y%d", message, i);
		sprintf(log_str, "user #%d tested the lower_case() function.\n", user);
		return log_str;
	}
	
	// Test Random Generation Command
	if(index(message, TestRandomGenerationCommand)) {
		sprintf(*answer, "\\wirand(22) -> { \\y%d, %d, %d, %d, %d, %d, %d, %d, %d, %d\\w }", irand(22), irand(22), irand(22), irand(22), irand(22), irand(22), irand(22), irand(22), irand(22), irand(22));
		sprintf(log_str, "user #%d tested the irand() function.\n", user);
		return log_str;
	}
	
	// Time Command
	if(index(message, TimeCommand)) {
		sprintf(*answer, "\\cSystem booted \\y%lu\\c seconds (%lu hours and %lu minutes) ago.", clock() / CLOCKS_PER_SEC, clock() / CLOCKS_PER_SEC / 60 / 60, ((clock() / CLOCKS_PER_SEC)/60) % 60);
		sprintf(log_str, "user #%d looked at the active sustem time.\n", user);
		return log_str;
	}
	
	// Type Delay Command
	if(index(message, TypeDelayCommand)) {
		sprintf(*answer, "\\wType Delay\\c is %s\n\\wtype rate = \\y%1.2f \\csec", (TYPE_DELAY ? "\\gOn" : "\\rOff"), TYPE_RATE);
		sprintf(log_str, "user #%d looked at the type delay state.\n", user);
		return log_str;
	}
	
	// VersionDisplayCommand
	if(index(message, VersionDisplayCommand)) {
		sprintf(*answer, "\\wSimple Bot\\c, version \\y%s", BOT_VERSION);
		sprintf(log_str, "user #%d asked for the version of the bot\n", user);
		return log_str;
	}
	// End of commands
	
	
	// Did the user send a special message ?
	switch( msg_type ) {
		case kDRAW: 
			sprintf(*answer, "Stop playing with that..");
			sprintf(log_str, "user #%d sent a drawing\n", user);
			return log_str;
			
		case kKICK: 
			sprintf(*answer, "La Vigie here. . .What do you think you are doing, now?? *nods*");
			sprintf(log_str, "user #%d tried to kick the bot\n", user);
			return log_str;
		
		case kSYSG: 
			sprintf(*answer, "Beware, I'm the god of this bbs.");
			sprintf(log_str, "user #%d sent a SysGuide message\n", user);
			return log_str;
		
		case kSYSO: 
			sprintf(*answer, "Beware, I'm the god of this bbs.");
			sprintf(log_str, "user #%d sent a SysOp message\n", user);
			return log_str;
	}
	
	// If we are here, that means that the message does not
	// contain any special command. So we free() the memory
	// allocated (because it will be allocated later, but 
	// with a more accurate size).
	free(*answer);
	
	
	// Lower case the input message.
	// We do this to avoid the CAPS LOCK users.
	lower_case(message);
	
	
	// French Dictionary parsing
	if( dictionary_parse(French_Dict, message, answer) ) {
	    sprintf(log_str, "user #%d spoke French\n", user);
		return log_str; 
	}
	
	
	// Spanish Dictionary parsing
	if( dictionary_parse(Spanish_Dict, message, answer) ) {
	    sprintf(log_str, "user #%d spoke Spanish\n", user);
		return log_str; 
	}
	
	
	// German Dictionary parsing
	if( dictionary_parse(German_Dict, message, answer) ) {
	    sprintf(log_str, "user #%d spoke German\n", user);
		return log_str; 
	}
	
	
	// Dutch Dictionary parsing
	if( dictionary_parse(Dutch_Dict, message, answer) ) {
	    sprintf(log_str, "user #%d spoke German\n", user);
		return log_str; 
	}
	
	
	// Italian Dictionary parsing
	if( dictionary_parse(Italian_Dict, message, answer) ) {
	    sprintf(log_str, "user #%d spoke Italian\n", user);
		return log_str; 
	}
	
	// Corpus parsing
	for(i=0; i<corpus_items; i++) {
		if( dictionary_parse(corpus[i], message, answer) ) {
	    	sprintf(log_str, "answered to user #%d\n", user);
			return log_str; 
		}
	}
	
	*answer = new char [MAX_ANS_LENGTH];
	if(not *answer) {
		warn("bot_reponse", __LINE__, "'new char []' failed");
		return NULL;
	}
	sprintf(*answer, "mm?");
	
	strcpy(log_str, DefaultLogStr);
	return log_str;
}


/* ***************************************************************** *
 * dictionary_parse                                                  *
 * ----------------                                                  *
 *   description                                                     *
 *     This function tests if the given message verifies the         *
 *     conditions of the current declaration. If it is the case,     *
 *     the corresponding answer is stored in the answer variable.    *
 *                                                                   *
 * ***************************************************************** */
int dictionary_parse(BLOC dict, char * message, char ** answer) {
	int     r = 0;
	
	if( condition_test( message, dict->condition ) ) {
		
		// If there's more than one sentence, I choose one randomly. 
		if( dict->nb_phrases > 1 ) {
			
			// Getting a (pseudo)random number..
			r = irand(dict->nb_phrases);
		
		} else {
			r = 0;
		}
		
		// I allocate enought memory to store the answer..
		*answer = new char [ strlen(dict->phrases[r]) ];
		
		if(not *answer) {
			warn("bot_reponse", __LINE__, "'new char []' failed");
			return NULL;
		}
		
		// .. and I copy the string. 
		strcpy(*answer, dict->phrases[r]);
		
		// I simulate the keystroke. 
		//type_on_keyboard( *answer ); // --> Currently disabled
		
		return 1;
		
	} else {
	    return 0;
	}
}


/* ***************************************************************** *
 * condition_test                                                    *
 * --------------                                                    *
 *   description                                                     *
 *     This function determines if the condition cond is verified,   *
 *     respecting the rules of the BNF Syntax.                       *
 *                                                                   *
 * ***************************************************************** */
int condition_test(char * str, CONDITION cond) {
	if( not cond ) {
		return 0;
		
	} elsif( cond->next ) {
		if( cond->op == AND ) {
			return index(str, cond->word) and condition_test(str, cond->next);
		
		} else if( cond->op == OR ) {
			return index(str, cond->word) or  condition_test(str, cond->next);
			
		} else {
			return 0;
		}
		
	} elsif( not cond->next ) {
		return index(str, cond->word);
		
	} else {
		return 0;
	}
	
	return 0;
}


/* ***************************************************************** *
 * condition_count                                                   *
 * ---------------                                                   *
 *   description                                                     *
 *     This function is used to count the number of linked           *
 *     conditions.                                                   *
 *                                                                   *
 * ***************************************************************** */
int condition_count(CONDITION cond) {
	if( not cond ) {
		return 0;
	} elsif( cond->next ) {
		return 1 + condition_count(cond->next);
	} else {
		return 1;
	}
}


/* ***************************************************************** *
 * init_bot                                                          *
 * --------                                                          *
 *   description                                                     *
 *     This function initializes the bot: it loads the dictionaries  *
 *     and the corpus, and some other things.                        *
 *                                                                   *
 * ***************************************************************** */
int bot_rep::init_bot(void) {
	char *		French_Dict_filename  = "french.dic";
	char *		Spanish_Dict_filename = "spanish.dic";
	char *		German_Dict_filename  = "german.dic";
	char *		Dutch_Dict_filename   = "dutch.dic";
	char *		Italian_Dict_filename = "italian.dic";
	char *		corpus_filename       = "corpus.dic";
	
	int     	i                     = 0;
	
	// Pseudo-Random Number Generator Initialization
	srand(time(NULL) ^ getpid());
	
	// Calculation of the size used by all the 
	// globals variables and constants
	globals_mem = sizeof(wchar) 
		+ sizeof(file) 
		+ sizeof(corpus) 
		+ sizeof(corpus_items)
		+ sizeof(French_Dict) * 5 
		+ sizeof(AND) * 2
		+ sizeof(used_mem) * 3
		+ sizeof(TYPE_RATE) 
		+ sizeof(TYPE_DELAY);
	
	const_mem = sizeof(const int) * 5
		+ sizeof(DefaultLogStr)
		+ sizeof(Confirm)
		+ (sizeof(const char *) + 1) * 13
		+ strlen(ActivateTypeDelayCommand)
		+ strlen(AuthorDisplayCommand)
		+ strlen(CommandsListingCommand)
		+ strlen(DataStatisticsCommands)
		+ strlen(DateOfCompilationCommand)
		+ strlen(DesactivateTypeDelayCommand)
		+ strlen(HelpDisplayCommand)
		+ strlen(ReloadCommand)
		+ strlen(TestRandomGenerationCommand)
		+ strlen(TimeCommand)
		+ strlen(TypeDelayCommand)
		+ strlen(VersionDisplayCommand);
	
	
	// French Dictionary
	if(not (file = G.GetKHFile(French_Dict_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open French dictionary");
		return errno;
	}
	wchar = next_useful_char();
	bloc(&French_Dict);
	fclose(file);
	
	// Spanish Dictionary
	if(not (file = G.GetKHFile(Spanish_Dict_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open Spanish dictionary");
		return errno;
	}
	wchar = next_useful_char();
	bloc(&Spanish_Dict);
	fclose(file);
	
	// German Dictionary
	if(not (file = G.GetKHFile(German_Dict_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open German dictionary");
		return errno;
	}
	wchar = next_useful_char();
	bloc(&German_Dict);
	fclose(file);
	
	// Dutch Dictionary
	if(not (file = G.GetKHFile(Dutch_Dict_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open Dutch dictionary");
		return errno;
	}
	wchar = next_useful_char();
	bloc(&Dutch_Dict);
	fclose(file);
	
	// Italian Dictionary
	if(not (file = G.GetKHFile(Italian_Dict_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open Italian dictionary");
		return errno;
	}
	wchar = next_useful_char();
	bloc(&Italian_Dict);
	fclose(file);
	
	// Corpus
	
	corpus = (BLOC *)malloc(sizeof(BLOC) * CORPUS_SIZE);
	//corpus = new BLOC[CORPUS_SIZE];  // This doesn't seem to work..
	if(not corpus) {
		warn("init_bot", __LINE__, "malloc() failed");
	}
	
	if(not (file = G.GetKHFile(corpus_filename, "r"))) {
		warn("init_bot", __LINE__, "can't open corpus");
		return errno;
	}
	wchar = next_useful_char();
	while( (i < CORPUS_SIZE) and bloc(&(corpus[i])) ) {
		i++;
	}
	corpus_items = i;
	fclose(file);
	return errno;
}


/* ***************************************************************** *
 * kill_bot                                                          *
 * --------                                                          *
 *   description                                                     *
 *     This function destroy all the dictionaries and the corpus.    *
 *                                                                   *
 * ***************************************************************** */
int bot_rep::kill_bot(void) {
	int 	i = 0;
	int 	j = 0;
	
	
	if(French_Dict) {
		// destroy French Dictionary
		delete_condition(French_Dict->condition);
		for(i=0; i<French_Dict->nb_phrases; i++) {
			free(French_Dict->phrases[i]);
		}
		free(French_Dict);
	}
	
	
	if(Spanish_Dict) {
		// destroy Spanish Dictionary
		delete_condition(Spanish_Dict->condition);
		for(i=0; i<Spanish_Dict->nb_phrases; i++) {
			free(Spanish_Dict->phrases[i]);
		}
		free(Spanish_Dict);
	}
	
	
	if(German_Dict) {
		// destroy German Dictionary
		delete_condition(German_Dict->condition);
		for(i=0; i<German_Dict->nb_phrases; i++) {
			free(German_Dict->phrases[i]);
		}
		free(German_Dict);
	}
	
	
	if(Dutch_Dict) {
		// destroy Dutch Dictionary
		delete_condition(Dutch_Dict->condition);
		for(i=0; i<Dutch_Dict->nb_phrases; i++) {
			free(Dutch_Dict->phrases[i]);
		}
		free(Dutch_Dict);
	}
	
	
	if(Italian_Dict) {
		// destroy Italian Dictionary
		delete_condition(Italian_Dict->condition);
		for(i=0; i<Italian_Dict->nb_phrases; i++) {
			free(Italian_Dict->phrases[i]);
		}
		free(Italian_Dict);
	}
	
	
	// destroy corpus
	for(i=0; i<corpus_items; i++) {
		if(corpus[i]) {
			delete_condition( (corpus[i])->condition );
			for(j=0; j<(corpus[i])->nb_phrases; j++) {
				free( (corpus[i])->phrases[j] );
			}
			free( corpus[i] );
		}
	}
	
	return errno;
}


/* ***************************************************************** *
 * delete_condition                                                  *
 * ----------------                                                  *
 *   description                                                     *
 *     This function destroy recursively the condition cond.         *
 *                                                                   *
 * ***************************************************************** */
void delete_condition(CONDITION cond) {
	if(cond != NULL) {
		delete_condition(cond->next);
		free(cond);
	}
}


/* ***************************************************************** *
 * next_char                                                         *
 * ---------                                                         *
 *   description                                                     *
 *     This function returns the next character (no filter is        *
 *     applied).                                                     *
 *                                                                   *
 * ***************************************************************** */
int next_char(void) {
	fflush(file);
	return getc(file);
}


/* ***************************************************************** *
 * next_useful_char                                                  *
 * ----------------                                                  *
 *   description                                                     *
 *     This function returns the next useful character (different    *
 *     from any space or control character).                         *
 *                                                                   *
 * ***************************************************************** */
int next_useful_char(void) {
	int c = 0;
	
	fflush(file);
	do {
		c = getc(file);
	} while(space(c));
	
	return c;
}


/* ***************************************************************** *
 * strip_comments                                                    *
 * --------------                                                    *
 *   description                                                     *
 *     This function skips the comments from the source file. A      *
 *     comment is everything from a sharp '#' till the end of the    *
 *     line.                                                         *
 *                                                                   *
 * ***************************************************************** */
int strip_comments(void) {
	while(wchar == '#') {
		do {
			wchar = next_char();
		} while( (wchar != 10) and (wchar != 13) );
		wchar = next_useful_char();
	}
	return 1;
}


/* ***************************************************************** *
 * bloc                                                              *
 * ----                                                              *
 *   description                                                     *
 *     This function analyses a block.                               *
 *                                                                   *
 * ***************************************************************** */
int bloc(BLOC * my_block) {
	int     n   = 0;
	int 	i   = 0;
	
	strip_comments();
	
	// Alocate a new bloc. 
	*my_block = new_bloc(NULL, 0, NULL);
	
	if(not *my_block) {
		return 0;
	}
	used_mem += sizeof(BLOC);
	
	if(condition(&((*my_block)->condition))) {
		strip_comments();  // Skip comments. 
		if(number(&n)) {
			(*my_block)->nb_phrases = n;
			strip_comments();  // Skip comments. 
			
			// I allocate enought memory to store the string array. 
			(*my_block)->phrases = (char **)malloc(sizeof(char *) * n);
			
			if( not (*my_block)->phrases ) {
				warn("bloc", __LINE__, "malloc() failed");
				return 0;
			}
			used_mem += sizeof(char *) * n;
			
			// I read the number of sentences I will read. 
			i = 0;
			while( (i < n) and (phrase(&((*my_block)->phrases[i]))) ) {
				wchar = next_useful_char();
				strip_comments();  // Skip comments. 
				i++;
			}
			
			// I test if there is a kick condition. 
			if( kick_at(&n) ) {
				(*my_block)->kick_at = n;
				strip_comments();  // Skip comments. 
				
				if( number(&n) ) {
					(*my_block)->nb_kick_phrases = n;
					strip_comments();  // Skip comments. 
					
					// I allocate enought memory to store the sentences array.
					(*my_block)->kick_phrases = (char **)malloc(sizeof(char *) * n);
					
					if( not (*my_block)->kick_phrases ) {
						warn("bloc", __LINE__, "malloc() failed");
						return 0;
					}
					used_mem += sizeof(char *) * n;
					
					// I read the number of sentences I will read. 
					i = 0;
					while( (i < n) and (phrase(&((*my_block)->kick_phrases[i]))) ) {
						wchar = next_useful_char();
						strip_comments();  // Skip comments. 
						i++;
					}
					
					return 1;
					
				} else {
					return 0;
				}
				
			} else {
			    (*my_block)->kick_at = 0;
			    (*my_block)->nb_kick_phrases = 0;
			    (*my_block)->kick_phrases = NULL;
			}
			
			return 1;
			
		} else {
			return 0;
		}
		
	} else {
		return 0;
	}
}


/* ***************************************************************** *
 * condition                                                         *
 * ---------                                                         *
 *   description                                                     *
 *     This function analyses a condition.                           *
 *                                                                   *
 * ***************************************************************** */
int condition(CONDITION * cond) {
	char *  	str = NULL;
	
	if(wchar == '(') {
		*cond = new_condition(NULL, 0, NULL);
		
		if(not *cond) {
			return 0;
		}
		used_mem += sizeof(CONDITION);
		
		wchar = next_useful_char();
		if(word(&str)) {
			if(wchar == ')') {
				(*cond)->word = str;
				wchar = next_useful_char();
				if(operator(wchar)) {
					(*cond)->op = wchar;
					wchar = next_useful_char();
					if(condition(&((*cond)->next))) {
						return 1;
					} else {
						return 0;
					}
				}
				return 1;
			} else {
				return 0;
			}
		} else {
			return 0;
		}
	} else {
		return 0;
	}
}


/* ***************************************************************** *
 * kick_at                                                           *
 * -------                                                           *
 *   description                                                     *
 *     This function analyses the kick statement.                    *
 *                                                                   *
 * ***************************************************************** */
int kick_at(int * limite) {
	
	if(wchar == 'k') {
		wchar = next_useful_char();
		
		// Read the limit at which the bot get nervous. (The function 
		// is actually not implemented, but is planed.)
		if(number(limite)) {
			
			if(wchar == ':') {
				wchar = next_useful_char();
				return 1;
			
			} else {
				return 0;
			}
			
		} else {
			return 0;
		}
		
	} else {
		return 0;
	}
	
	limite = 0;
	return 0;
}


/* ***************************************************************** *
 * phrase                                                            *
 * ------                                                            *
 *   description                                                     *
 *     This function tries to recognize a sentence.                  *
 *                                                                   *
 * ***************************************************************** */
int phrase(char ** str) {
	char    buf[MAX_BUF];
	int     i   = 0;
	
	if(not buf) {
		warn("phrase", __LINE__, "'new char []' failed");
		return 0;
	}
	
	if(wchar == '"') {
		wchar = next_char();
		while(wchar != '"') {
			buf[i++] = wchar;
			wchar = next_char();
			
			//Here, I take care of characters protected with backslash. 
			if(wchar == '\\') {
				wchar = next_char();
				
				// double quote '\"'
				if(wchar == '"') {
					buf[i++] = '"';
					wchar = next_char();
					
				// end of line '\n'
				} elsif(wchar == 'n') {
					buf[i++] = '\n';
					wchar = next_char();
					
				// default case: blackslash
				} else {
					buf[i++] = '\\';
				}
			}
		}
		buf[i] = 0;
		// I allocate the memory needed to contain the whole string...
		*str = new char [i+1];
		if(*str != NULL) {
			used_mem += sizeof(char)*(i+1);
			// ... and I copy it in this this variable
			strcpy(*str, buf);
			return 1;
		} else {
			warn("phrase", __LINE__, "'new char []' failed");
			return 0;
		}
	} else {
		return 0;
	}
}


/* ***************************************************************** *
 * word                                                              *
 * ----                                                              *
 *   description                                                     *
 *     This function tries to recognize a word.                      *
 *                                                                   *
 * ***************************************************************** */
int word(char ** str) {
	char  	buf[MAX_BUF];
	int 	i   = 0;
	
	if(not buf) {
		warn("word", __LINE__, "'new char []' failed");
		return 0;
	}
	
	if(letter(wchar)) {
		while(letter(wchar) || (wchar == 32)) {
			buf[i++] = wchar;
			wchar = next_char();
			
			// Here, I take care of parenthesises protected with backslash.
			if(wchar == '\\') {
				wchar = next_char();
				
				if(wchar == ')') {
					buf[i++] = ')';
					wchar = next_char();
				} else {
					buf[i++] = '\\';
				}
			}
		}
		buf[i] = 0;
		// I allocate the memory needed to contain the whole string...
		*str = new char [i+1];
		if(*str != NULL) {
			used_mem += sizeof(char) * (i+1);
			// ... and I copy it in this this variable
			strcpy(*str, buf);
			return 1;
		
		} else {
			warn("word", __LINE__, "'new char []' failed");
			return 0;
		}
	
	} else {
		return 0;
	}
}


/* ***************************************************************** *
 * number                                                            *
 * ------                                                            *
 *   description                                                     *
 *     This function tries to recognize a number, and puts it in     *
 *     the variable a if it is the case.                             *
 *                                                                   *
 *   explanation                                                     *
 *     To calculate the number from its digits separatly read in     *
 *     from left to right, we use the decomposition in base 10 of    *
 *     the number:                                                   *
 *                     k          k-1                                *
 *         a  =  a . 10  +  a . 10  +  ...  +  a . 10  +  a          *
 *                k          k-1                1          0         *
 *                                                                   *
 *     Furthermore, as the calculation of the number is also done    *
 *     from left to right, it's really easy (and fast) to calulate   *
 *     a number with Horner formula.                                 *
 *                                                                   *
 * ***************************************************************** */
int number(int *a) {
	if(digit(wchar)) {
		*a = 0;
		while(digit(wchar)) {
			*a = *a * 10 + value(wchar);
			wchar = next_useful_char();
		}
		return 1;
	} else {
		return 0;
	}
}


/* ***************************************************************** *
 * new_condition                                                     *
 * -------------                                                     *
 *   description                                                     *
 *      This function creates a new condition.                       *
 *                                                                   *
 * ***************************************************************** */
CONDITION new_condition(char * word, char op, CONDITION next) {
	CONDITION	ptr = (CONDITION)malloc(sizeof(condition_t));
	
	if(ptr != NULL) {
		ptr->word = word;
		ptr->op   = op;
		ptr->next = next;
		used_mem += sizeof(condition_t);
	} else {
		warn("new_condition", __LINE__, "malloc() failed");
	}
	return ptr;
}


/* ***************************************************************** *
 * new_bloc                                                          *
 * --------                                                          *
 *   description                                                     *
 *      This function creates a new block.                           *
 *                                                                   *
 * ***************************************************************** */
BLOC new_bloc(CONDITION condition, int nb_phrases, char ** phrases) {
	BLOC	ptr = (BLOC)malloc(sizeof(bloc_t));
	
	if(ptr != NULL) {
		ptr->condition  = condition;
		ptr->nb_phrases = nb_phrases;
		ptr->phrases    = phrases;
		used_mem += sizeof(bloc_t);
	} else {
		warn("new_bloc", __LINE__, "malloc() failed");
	}
	return ptr;
}


/* ***************************************************************** *
 * lower_case                                                        *
 * ----------                                                        *
 *   description                                                     *
 *      This function converts the string str to lower case and      *
 *      returns the length of the string.                            *
 *                                                                   *
 * ***************************************************************** */
int lower_case(char * str) {
	int i;
	int l = strlen(str);
	const char shift = 'a' - 'A'; // This won't work with EBCDIC, but I don't care :-)
	
	for(i=0; i<l; i++) {
		if((str[i] >= 'A') and (str[i] <= 'Z')) {
			str[i] += shift;
		}
	}
	return i;
}


/* ***************************************************************** *
 * irand                                                             *
 * -----                                                             *
 *   description                                                     *
 *      This function return a random integer.                       *
 *                                                                   *
 * ***************************************************************** */
int irand(int n) {
	return rand() % n;
}


/* ***************************************************************** *
 * warn                                                              *
 * ----                                                              *
 *   description                                                     *
 *      This perlish function prints a warning message.              *
 *                                                                   *
 * ***************************************************************** */
int warn(char * function, int line, char * message) {
	return fprintf(stderr, "*** %s() [in %s, line %d] *** Error %d: %s\n", 
		function, __FILE__, line, errno, message);
}


/* ***************************************************************** *
 * die                                                               *
 * ---                                                               *
 *   description                                                     *
 *      This perlish function print an fatal error message, and      *
 *      then exit the program.                                       *
 *                                                                   *
 * ***************************************************************** */
int die(char * function, int line, char * message) {
	fprintf(stderr, "*** %s() [in %s, line %d] *** Error %d: %s\n", 
		function, __FILE__, line, errno, message);
	exit(errno);
	return 1;
}

