/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#include "Aff.H"
#include "consts.H"
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <termios.h>
#include <termio.h>
#include "G_Path.H"
#include "Debug.H"

extern G_Path G;
  
static const char * repmsg = "msg/";  // rep des msgs

Aff::Aff(void)
  :MemAttach(CLEMSG,sizeof(messages)),MsgChemin(0),langue(1),NbSav(0),T_more(0),nblignes(0)
{
  m = (messages *)MEM;
  // Put no echo, no blocking, and all the stuff
  termios termtype;
  
  tcgetattr(STDIN_FILENO,&termtype);
  oldCtrl=termtype.c_lflag;
  oldMin=termtype.c_cc[VMIN];
  termtype.c_lflag &=!(ECHO|ICANON);
  termtype.c_cc[VMIN]=1;
  tcsetattr(STDIN_FILENO,TCSANOW,&termtype);
  // Et hop, c'est fait :)
}

Aff::~Aff(void)
{
  FreeMsg();
  delete [] MsgChemin;
  // Restore term
  termios termtype;
  tcgetattr(STDIN_FILENO,&termtype);
  termtype.c_lflag = oldCtrl;
  termtype.c_cc[VMIN] = oldMin;
  tcsetattr(STDIN_FILENO,TCSANOW,&termtype);
}

void Aff::FreeMsg()
{
  for (int i=0; i<NbSav;i++) delete [] sav[i];
  NbSav=0;
}

void Aff::GetPasswd(int i, char * c)
{
  printnum(i);
  strcpy(c,(char *)getpass(" "));
}

void Aff::NewMore(void)
{
  nblignes = 0;
}

void Aff::Pause(void)
{
  printnum(M_Pause);  GetCh();
  ln();
}

void Aff::AddMore(int i)
{
  nblignes += i;
  if ( T_more && nblignes >= T_more ) Pause();
}

char * Aff::GetNum(int i)
{ return colorize(m->msg+m->dpl[num][i]); }

void Aff::printnum(int i, ...)
{
  va_list ap;

  char temp[500];
  va_start(ap, i);
  vsprintf(temp, GetNum(i), ap);
  Direct(temp);
  va_end(ap); 
}

void Aff::DirectMSG(const char * a)
{
  Direct(GetStringColor(a));
}

void Aff::ln(void) 
{ Direct("\n"); }

void Aff::ln2(void) 
{ Direct("\n\n"); }

void Aff::beep(void) 
{ Direct("\a");  }

void Aff::Direct(const char * a)
{ 
  if (! a)  return;
  const char * p = a;
  const char * p2 =a;
  while ( * p )
    {
      while ( *p && (*p !='\n') ) p ++;
      if ( write(1,p2,p-p2) == -1 )
	{
	  perror("Aff::Direct()");
	  throw NoScreen();
	}
      p2=p+1;
      if ( * p )
	{
	  p ++; 
	  write(1,"\n",strlen("\n"));
	  AddMore(1);
	}
    }
}

void Aff::Change_Droit(unsigned short can,unsigned short & dr, char c)
{
  unsigned short i = 1 << ( c-'0');
  if ( can & i)
    dr ^= i;
  else
    printnumln(M_REVE);
}

void Aff::Direct(char c)
{
  write(1,&c,1);
}

void Aff::PrintIfOk(int i, const String & a)
{
  if ( ! a.null() ) printnumln(i,a.data());
}

void Aff::PrintIfOk(const char * i, const String & a)
{
  if ( ! a.null() ) printf2(i,a.data());
}

char Aff::bille(int I)
{ return(I?'*':' '); }

char Aff::croix(int I)
{ return(I?'X':' '); }

char * Aff::videC(const String & a)
{ return(colorize(a.data())); }

// ask confirmation if needed
int Aff::Confirm(int nummsg, int option)
{  
  ln();
  printnum(nummsg);
  printnum(M_Question);

  int ch;
  try
    {
      
      do
	{ 
	  if (  flag & option ) 
	    ch='o';
	  else
	    ch = GetCh();
	} while ( ch != 'o' && ch != 'n' && ch != 's' && ch != 'y' && ch != 'j' );
      ch = ch != 'n';
      printnumln(M_No +  ch );
    }
  catch ( Cancel ) 
    {
      return 0;    
    }
  
  
  return ch;
}

void Aff::printnumln(int i, ...)
{
  va_list ap;

  char temp[500];
  va_start(ap, i);
  vsprintf(temp, GetNum(i), ap);
  Direct(temp);
  va_end(ap); 

  ln();
}

void Aff::printheure(int i,time_t t)
{
  tm * HEURE = localtime(&t);
  
  printnum(i, 
    HEURE->tm_mday, HEURE->tm_mon+1, HEURE->tm_year+1900,
    HEURE->tm_hour, HEURE->tm_min);  
}

void Aff::Droits(unsigned short f, int valid)
{
 if ( valid && ! ( f & D_untwit ) ) printnum(M_NotValid); 
 int i;
 for (i=0; i< 12; i++ )
   if ( f & ( 1 << i ) ) printnum(M_ValidUser+i);
 ln();         
}

void Aff::Droits_Croix(unsigned short can,unsigned short f)
{
 int i;
 for (i=0; i< 12; i++ ) if (can & ( 1<<i ) )
   {
     char c = '0'+i;
     Direct(c);
     Direct(" - [");
     Direct(bille(f & ( 1 << i )));
     Direct("] - ");
     printnum(M_ValidUser+i);
     ln();
   }
}

void Aff::printdepuis(time_t t)
{
  time_t Diff = time(0) -t;
  int i;

  if (Diff < 60 )            i=M_TpsSecondes;  else
  if ( (Diff /=  60 ) < 60 ) i=M_TpsMinutes;   else
  if ( (Diff /= 60 ) < 24 )  i=M_TpsHeures;    else
  if ( (Diff /= 24 ) < 30 )  i=M_TpsJours;     else
               { Diff /= 30; i=M_TpsMois; }
  
  printnum(M_DepuisTps,Diff, GetNum(i)); 
}

void Aff::Reset(unsigned short L_langue,unsigned long L_flag, unsigned short newmore) 
{
  char * temp;

  langue=L_langue; flag=L_flag;
  FreeMsg();
  num=-1;
  for (unsigned int i=0; i<G.Nblg();i++) if (1<<i == langue) num=i;
  if (num==-1)
  { Debug(1,"Aff::Reset():Language error"); num=0; }
  if (MsgChemin) 
    delete [] MsgChemin;

  temp=G.Lg(num);
  MsgChemin = new char[strlen(repmsg)+strlen(temp)+2];
  sprintf(MsgChemin,"%s%s",repmsg,temp);

  none  = String(GetNum(M_InfoVide ));
  hidden= String(GetNum(M_InfoCache));
  T_more = newmore;
}

char * Aff::colorize(const char * a)  // colorize a string
{
  int nbcars=0;
  int taille=strlen(a);
  char * temp, * res;
  const char * t2=a;
  for(int i=0;i< taille;i++) if (a[i]=='\\') nbcars++;

  res=temp = new char[taille+nbcars*6+5];
  temp[0]=0;

  while (*t2)
  {
    if (* t2=='\\')
      {
	t2++;
	char * poal = ColourChar(* t2);
	strcpy(temp,poal);
	temp += strlen(poal);
      }
    else
      {
	temp[0]=*t2; temp[1]=0;
	temp++;
      }
    t2++;
  }
  //  printf("P\n");
  //printf("%d done\n",temp-res);
  //Direct(res);
  return PUSH(res);
}

char * Aff::PUSH(char * i)
{
  if ( NbSav >= 99 ) FreeMsg();
  sav[NbSav++]= i;
  return i;
}

char * Aff::GetStringColor(const char * a)
/* Donne la chaine de cars corrspondant a notre constante */
{
  int taille;
  char * temp2, * temp3, * result;
  FILE * f;

  temp3 = new char[strlen(MsgChemin)+strlen(a)+4];

  sprintf(temp3,"%s/%s",MsgChemin,a);
  temp2 = G.GetEtcHome(temp3);
  delete [] temp3;

  if ((f=fopen(temp2,"r")))
  {
    fseek(f,0,SEEK_END);
    taille = ftell(f);
    rewind(f);
    temp3 = new char[taille];
    fread(temp3,taille,1,f); 
    temp3[taille-1]=0;
    fclose(f);
  } else
  {
    char * msg="\\rCouldn't open \\y%s\\r, post this in BUGS !\n\\a";
    temp3=new char[strlen(msg)+strlen(temp2)+1];
    sprintf(temp3,msg,temp2);
  }

  result=colorize(temp3);
  delete [] temp3;
  delete [] temp2;

  return result;
}

int Aff::PrintFile(char * Nom, int cry)
  // Imprime le fichier demande
{
  int taille;
  char  * temp3, * result;
  FILE * f;

  if ((f=fopen(Nom,"rb")))
  {
    fseek(f,0,SEEK_END);
    taille = ftell(f);
    rewind(f);
    temp3 = new char[taille+4];
    fread(temp3,taille,1,f);
    temp3[taille]=0;
    fclose(f);
  } else
  {
    if ( ! cry ) return 0;
    char * msg="\\rCouldn't open \\y%s\\r, post this in BUGS !\n\\a";
    temp3=new char[strlen(msg)+strlen(Nom)+1];
    sprintf(temp3,msg,Nom);
  }

  result = colorize(temp3);

  delete [] Nom;
  delete [] temp3;

  Direct(result);
  return 1;
}

void Aff::printf2(const char * a, ...)
/* printf2 renvoie la chaine de cars colorisee s'il le faut 
correspondant a la constante de type chaine */
{
  va_list ap;

  va_start(ap, a);
  char  temp[15000];
  vsprintf(temp, GetStringColor(a), ap);
  Direct(temp);
  va_end(ap);
}
char truc[2];

char * Aff::ColourChar(char key)
/* a partir d'un char, reconstitue la couleur a avoir */
{
  char * s;

  switch(key)
  {
    case 'd': s="\033[30m"; break; /* dark textcolor	*/
    case 'r': s="\033[31m"; break; /* red textcolor	*/
    case 'g': s="\033[32m"; break; /* green textcolor	*/
    case 'y': s="\033[33m"; break; /* yellow textcolor	*/
    case 'b': s="\033[34m"; break; /* blue textcolor	*/
    case 'p': s="\033[35m"; break; /* purple textcolor	*/
    case 'c': s="\033[36m"; break; /* cyan textcolor	*/
    case 'w': s="\033[37m"; break; /* white textcolor	*/
    case 'D': s="\033[40m"; break; /* dark backgroundcolor  */
    case 'R': s="\033[41m"; break; /* red backgroundcolor   */
    case 'G': s="\033[42m"; break; /* green backgroundcolor */
    case 'Y': s="\033[43m"; break; /* yellow backgroundcolor*/
    case 'B': s="\033[44m"; break; /* blue backgroundcolor  */
    case 'P': s="\033[45m"; break; /* purple backgroundcolor*/
    case 'C': s="\033[46m"; break; /* cyan backgroundcolor  */
    case 'W': s="\033[47m"; break; /* white backgroundcolor */
    case 'a': s="\033[0m";  break; /* 'reset'-attribute	    */
    case 'u': s="\033[4m";  break; /* 'underlined'-attribute*/
    case 'f': s=((flag & U_bold )?"":"\033[1m");  break; /* 'bold'-attribute      */
    case 'e': s=((flag & U_flash)?"":"\033[5m");  break; /* 'flashing'-attribute  */
    case 'i': s="\033[7m";  break; /* 'hidden'-attribute	*/
    default: truc[0]=key; truc[1]=0; return(truc);   
  }
  return(flag & U_ansi?s:"");
}

void Aff::Blank(void)
{
  Direct(ColourChar('a'));
}

char * Aff::HiddenC(const String & c, int i, int k)
{
  char * temp = new char[strlen(c.data())+50];
  if ( i ) // hidden
    {
      if ( k ) // can read
	sprintf(temp, "%s %s",hidden.data(),colorize(c.data()));
      else // can't read
	sprintf(temp,"%s",hidden.data());
    }
  else
    sprintf(temp,"%s",colorize(c.data()));
	
  return PUSH(temp);
}

char * Aff::HiddenN(int i, int j, int k)
{
  char * temp =  new char[50];
  sprintf(temp,"%d",i);

  PUSH(temp);
  return HiddenC(temp,j,k);
}

void Aff::Efface(void)
{
  if (flag & U_ansi) Direct("\033c");
}

void Aff::DelCar(void)
{Direct("\b \b"); }

char Aff::GetCh(void)
{
  char c;
  NewMore();
  if ( read(0,&c,1) <=0 )
    {
      perror("Aff::GetCh()");
      throw NoClav();
    }
  if ( c == 3 ) throw Cancel();
  
  return c;
}




