/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#ifndef __C_USER_C__
#define __C_USER_C__

#include "G_B_R_Pipe.H"
#include "G_User.H"
#include "MAX.H"
#include <time.h>

//class PlusPersonne {};   // If there seem to be no server
class ErreurLecture {};  // If can not read data from server

class Un_X // contains a message sent by an user to you.
{
public:
  unsigned char type;   // 'X' = 'X' message. 'd' = drawing, ...
  unsigned short ki;    // usernumber of who sent it
  unsigned short num;   // number of system message, if type='*'
  unsigned short right; // possibly, they can be a right attached to an X
  char * buf;           // message if type !='d' or '*'.
                        // if type='*', NULL, if type='d' buf[0]=key
  Un_X() :type(0),ki(0),num(0),right(0),buf(0)
    {}
  ~Un_X() 
    {  if (buf) delete [] buf; }
};

class Un_Roume  // contains a room info
{
public:
  unsigned short numroom, pos, maxpos; // number of room, actual pos, max post
  Un_Roume()
    :numroom(0),pos(0),maxpos(0)
    {}
};


class Un_Post   // contains a post ( message in a room )
{
public:
  Un_Roume room;  // room info
  unsigned char TYPE_ROOM; // what is it ( X-msg, mail, post, ... )
  String sujet, alias, post; 
  unsigned short ki, move_u, move_r;   // who did the post, who moved it, from where
  unsigned short right;                // right assigned to the post
  unsigned short reply_r, reply_n;     // room of the reply, number of the post
  String reply_s;  // subject of the reply
  String reply_u;  // who made the first post

  unsigned char type;  // post in room 0 ( x-log ) type of x-msg
  unsigned short * Ki; // to who was sent the post ( mail, x-log )
  unsigned char * Koa; // what are the status ( N = not Read, ... )
  unsigned short nb;   // number of persons involved for mail, x-log
  time_t date;         // when was the post made
  Un_Post()
    :ki(0),move_u(0),move_r(0),
    right(0),
    type(0),Ki(0),Koa(0),nb(0),date(0)
    {}
  ~Un_Post()
    { 
      if ( Ki ) delete [] Ki; 
      if ( Koa ) delete [] Koa;
    }
};

class Un_Ack  // Contains an acknoledgment
{
public:
  unsigned char type;  // message received
  unsigned short ki;   // it was sent to user ki
  int received;        // if false, the user logged of before receiving it
  Un_Ack()  
    :type(0),ki(0),received(0)
    {}
};

class Un_log // a user logged in or out
{
public:
  int in;            // if true, in, else out
  unsigned short ki; // usernumber
  Un_log()
    :in(0),ki(0) {} 
};

class Modified{};    // your info have been modified, and have been reloaded

class Hello_Baille   // server says hello or baille
{
public:
  int hello;         // = 1 means hello
  Hello_Baille(int i)
    :hello(i)
    {}
};

class RListe         // server sends you the list of rooms
{
public:
  LISTE * Z, * K, * L,* E,* N;
  // Zapped, kicked, Read, being read, non read
  RListe ()
    {
      Z = new LISTE(MAX_ROOM);
      K = new LISTE(MAX_ROOM);
      L = new LISTE(MAX_ROOM);
      E = new LISTE(MAX_ROOM);
      N = new LISTE(MAX_ROOM);
    }
  ~RListe()
    {
      delete Z; delete K; delete L; delete E; delete N;
    }
};

class Un_Tag  // Tag operation
{
public:
  unsigned char type;
/*
  N = not available  ( not enough users )
  R = received
  S = Sent
  F = Woke up, flap flap comming
  L = lost ( too idle ), someone took it
  E = Error, Coco is on someone else 
  e = not a player, can not play */
  time_t Date;           // updated sometimes ( if received or Sent I think )
  unsigned short ki;     // who did the action, if need to know
  Un_Tag()
    :type(0),Date(0),ki(0)
    {}
};

class Un_Medor // Medor operation
{
public:
  unsigned char type; // par exemple !
  Un_Medor()
    :type(0)
    {
    }
};

class C_User_C: public G_User, public  G_B_R_Pipe
{
public:
  C_User_C(const char *);
  ~C_User_C(void);

  void Lit(unsigned short & a) { G_B_R_Pipe::Lit(a); }
  void Lit(unsigned char & a) { G_B_R_Pipe::Lit(a); }
  void Lit(char * & a) {G_B_R_Pipe::Lit(a); }
  void LitMsg(unsigned char & a, unsigned short & b) { G_B_R_Pipe::LitMsg(a,b); }
  void Lit(String & a){G_B_R_Pipe::Lit(a); }
  void Lit(void * a, unsigned int b) { G_B_R_Pipe::Lit(a,b); }

  // Messages to send to server
  void Baille(void);     // asks the server to log out
  void SendTag(void);    // send Coco
  void SendMedor(void);  // play with Medor
  void SendX(const LISTE & a,unsigned char koa, const char * Message, unsigned short);
  void Liste_Roumes(void); // ask the server to send the list of rooms
  void Reboot(void);       // ask the server to reboot
  void LoadMsg(void);      // ask the server to reload messages
  void LoadConfig(void);   // ask the server to reload configuration
  void Ack(unsigned short ki, unsigned char type); // acknodge a msg
  void Mail(const LISTE & a, const char * post, int reply); // post a mail
  void Poste(unsigned short NumRoom, const char * subject, const char * alias,
	     const char * post, int reply, unsigned short );
  void DeletePost(unsigned short NumRoom, unsigned short pos);
  void GetNextPost(unsigned short NumRoom); // Ask the server to send next post
  void SetPosRoom(unsigned short NumRoom, unsigned short nvellepos);
  void ZapRoom(LISTE & ); // List of rooms to zap
  void Jump_To_Room(unsigned short room, unsigned char dir);
  void CopierPost(unsigned short ,unsigned short , 
			  unsigned short );
  
  //  void Gere(void); Peut pas marcher, cf. bug g++

  // various data
  unsigned short DERNIER, DERNIERDERNIER,SG; // last sender, 
                                             // before last sender, sysguide
  time_t CocoDate;
public: // will be provate when g++ bug corrected
  void Get_Log(int, Un_log &);
  void Get_Ack(int , Un_Ack &);
  void Get_X(Un_X & );
  void Get_RListe(RListe & );
  void Get_Tag(Un_Tag &);
  void Get_Medor(Un_Medor &);
  void Get_Room(Un_Roume & );
  void Get_Post(Un_Post &);
};

#endif

