/* ***************************************************************** *
 * Login - General utility functions                                 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Tue 01 Sep 1998 *
 * Modification date . . . . . . . . . . . . . . . . Mon 07 Dec 1998 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standard includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>


/* ***************************************************************** *
 * custom includes                                                   *
 * ***************************************************************** */
#include "utils.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    1024


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char         program[]               = "Login/utils";



/* ***************************************************************** *
 * print_file()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function prints the given file to standard output.         *
 *                                                                   *
 * ***************************************************************** */
int print_file(char * filename) {
    FILE *  file = NULL;
    char    buf[BUF_SIZE];
    int     size = 0;
    int     n = 0;
    
    file = fopen(filename, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
                "%s: in send_file(): can't open '%s': error %d: %s\n", 
                program, filename, errno, strerror(errno)
        );
        return 0;
    }
    
    setbuf(file, NULL);
    
    while((n = fread(buf, sizeof(char), BUF_SIZE, file)) != 0) {
        fwrite(buf, sizeof(char), n, stdout);
        size += n;
    }
    
    return size;
}


/* ***************************************************************** *
 * lower_case()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function lower cases the given string.                     *
 *                                                                   *
 * ***************************************************************** */
int lower_case(char * str) {
    char *  s = str;
    int     i = 0;
    
    while(*s != 0) {
        *s = tolower(*s);
        s++;
        i++;
    }
    
    return i;
}


/* ***************************************************************** *
 * match()                                                           *
 * -----                                                             *
 * description                                                       *
 *   This function searches throught the given string the given      *
 *   pattern. The pattern can contains joker characters: the star    *
 *   (*) character matches zero or more characters.                  *
 *                                                                   *
 * explanation                                                       *
 *   As I didn't know how to write a good pattern matching function  *
 *   (neither to find a good algorithm) I created this simple        *
 *   algorithm, which seems quite correct.                           *
 *   Its main principle is to divide the pattern on the star         *
 *   characters, and to make each part (which is constant) match     *
 *   with his supposely corresponding part of the string by simply   *
 *   using strncmp(). This could be understood as below:             *
 *                                                                   *
 *                            foo*bar                                *
 *                            --- ---                                *
 *                     ________/   \__________________               *
 *                    /                               \              *
 *                   ---                             ---             *
 *      My God! This fool speaks once again with the bartender.      *
 *                                                                   *
 *   As you could doubt, I didn't mathematically prove that this     *
 *   algorithm works! I simply verified that it worked with some     *
 *   examples (as the one above). Hey, I am poor programmer ;-)      *
 *                                                                   *
 * ***************************************************************** */
int match(char * pattern, char * string) {
    char *  s = NULL;
    int     n = 0;
    
    s = strchr(pattern, '*');
    
    if(s != NULL) {
        n = s - pattern;
        return (strncmp(string, pattern, n) == 0) && match(s+1, string+n);
        
    } else {
        return strstr(string, pattern) != NULL;
    }
    
    return 0;
}


/* ***************************************************************** *
 * chomp()                                                           *
 * -----                                                             *
 * description                                                       *
 *   This function erases the end-of-line char of the given string   *
 *   if needed.                                                      *
 *                                                                   *
 * ***************************************************************** */
int chomp(char * str) {
    char    c = 0;
    int     len = strlen(str);
    
    c = str[len - 1];
    
    if(c == CR || c == LF) {
        str[len - 1] = 0;
        return 1;
        
    } else {
        return 0;
    }
}


/* ***************************************************************** *
 * star()                                                            *
 *                                                                   *
 * ***************************************************************** */
char star(int needed) {
    return needed ? '*' : ' ';
}


/* ***************************************************************** *
 * cross()                                                           *
 *                                                                   *
 * ***************************************************************** */
char cross(int needed) {
    return needed ? 'X' : ' ';
}


/* ***************************************************************** *
 * empty()                                                           *
 *                                                                   *
 * ***************************************************************** */
char * empty(char * str) {
    return str ? str : "";
}


