/*
   Copyright (C) 1992 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

// Analyse.cpp: implementation of the Analyse class.
//
//////////////////////////////////////////////////////////////////////

#ifdef _WINDOWS_
#include "stdafx.h"
#include "Bimbo.h"
#else
#include <ctype.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#endif

#include "Analyse.h"
#include "ErrAnalyse.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define NONTERMINATEUR(c) ((c)!='\n'&&(c)!='\0')

int currentline;

enum type_token { CHAINE, NB, LISTENB, LISTECH };

Analyse::Analyse()
:firstlistenb(0),firstchar(0),firstnb(0),firstlistestring(0)
{	
  currentline = 0;
}

Analyse::~Analyse()
{
	deleteTokens((Token *)firstchar);
	deleteTokens((Token *)firstnb);
}

Token * Analyse::chercheToken(Token *first, const char *name)
{
	while ( first )
	{
		if ( first->is(name) ) return first;
		first = first ->getNext();
	}
	return 0;
}

void Analyse::addToken(Token ** first, Token *add)
{
	Token ** previous =first;
	while ( * first ) { first = &((*first)->next); previous=first; }
	if ( * previous )
	{
		(* previous) -> next = add;
		add -> previous = * previous;
	}
	else
	{
		* first = add;
		add->previous = 0;
	}
}

char Analyse::getName()
{
	return msgname;
}

char Analyse::getType()
{
	return msgtype;
}

void Analyse::ignoreSpace()
{
	while ((*p==' ')||(*p=='\t')) p++;			 // ignoring spaces
}

void Analyse::Expect(char a)
{
	ignoreSpace();
	if ( *p != a )
	{
		char msg[100];
		sprintf(msg,"expected '%c' and found '%c'",a,*p);
		throw new ErrAnalyse(msg);
	}
	p++;
}

void Analyse::deleteTokens(Token *first)
{
	if ( first ) deleteTokens(first->next);
	delete first;
}

void Analyse::ExpectReturn()
{
	ignoreSpace();
	if ( *p != '\n' )
	{
		throw new ErrAnalyse("Expected a linefeed");
	}
	while ( *p == '\n' ) 
	{
	   p++;
	   currentline++;
	}
	ignoreSpace();
}

char Analyse::ExpectAlpha()
{
	ignoreSpace();
	if ( !isalpha(*p) )
		throw new ErrAnalyse("Expected alpha caracter");
	return *(p++);
}

char * Analyse::go(char * buff)
{
	char *p2;
	
	p=buff;
#ifdef _WINDOWS_
	MessageBox(NULL,p,"DATA",NULL);
#endif
			
	// first, we want something. with spaces possibly anywhere,
	// like that: x(x)\n {
    msgtype = ExpectAlpha();	
	Expect('(');
	msgname = ExpectAlpha();	
	Expect (')');		
    ExpectReturn();	
	Expect('{');
	ExpectReturn();

	// now, the lines are:
	// string = "value"
	// $string = "value"
	// #number = value
	// @listnumbers = value1,value2,value3
	// %listechaine ="hgf" "hgf" "fg"
	// spaces or tabs possibly anywhere

	do 
	{		
		type_token tt; // default = string
		
		// try to see what is the type of the
		// value
		switch ( *p )
		{
		case '$':
			tt = CHAINE;
			p++;
			break;
		case '@':
			tt = LISTENB;
			p++;
			break;
		case '#':
			tt= NB;
			p++;
			break;
		case '%':
			tt = LISTECH;
			p++;
			break;
		default:
			tt= CHAINE;
			// default: string, and do not go to next car
		}

		p2=p; 
		while (isalnum(*p2)) p2++; // getting token name
		
		char * debutnom = p;
		char * finnom   = p2;					
		
		p=p2;
		
		Expect('=');				
		ignoreSpace();

		// We are at the begining of the token
		switch ( tt ) 
		{
		case CHAINE:
			{								
				charToken * t = new charToken(debutnom,finnom-debutnom); // create a new char token
				t->setValue(getNextString()); 
				addToken((Token **)(&firstchar),t);				
			} // end case CHAINE
			break;
		case NB:
			{						
				nbToken * t = new nbToken(debutnom,finnom-debutnom); // create a new number token
				t->setValue(getNextNumber());
				addToken((Token **)(&firstnb),t);				
			} // end case NB
			break;
		case LISTENB:
			{
				listenbToken * t = new listenbToken(debutnom,finnom-debutnom); // create a new list of number token
				while ( NONTERMINATEUR(*p) ) t->addNumber(getNextNumber());
				addToken((Token **)(&firstlistenb),t);
			} // end case LISTNB
			break;
		case LISTECH:
			{
				listeStringToken * t = new listeStringToken(debutnom,finnom-debutnom); // create a new list of number token				
				while ( NONTERMINATEUR(*p) ) t->addString(getNextString());
				addToken((Token **)(&firstlistestring),t);
			}
			break;
		} // end case type		

		ExpectReturn();		
	} while ( *p != '}' );

	p++;
	ExpectReturn();                  // return
	
	// either it is finished, or there is an extra message
	if ( ! *p ) 
	{
		delete [] buff; // free the buffer
		return 0;       // finished
	}
	
	// there is perhaps an extra message after this one
	char * newbuff = new char[strlen(buff) - (p-buff) + 1 ]; // give back the message
    strcpy(newbuff,p);
    delete [] buff;
	return newbuff;
}

const char * Analyse::getChar(const char * a)
{
	charToken * poil;
	if ((poil=(charToken *)chercheToken(firstchar,a))) return poil->getValue();
	return 0;
}

int Analyse::getNumber(const char * a)
{
	nbToken * poil;
	if ((poil=(nbToken *)chercheToken(firstnb,a))) return poil->getValue();
	return 0;
}


int Analyse::getNextNumber()
{
	char * p2 = p;	
	while (isdigit(*p)) p++;
	if ( p2 == p )
	{
		if ( *p == '\n' ) return 0; // nothing means 0
		throw new ErrAnalyse("Expected a number");
	}
	char * temp = new char[p-p2+1];
	strncpy(temp,p2,p-p2);
	temp[p-p2]=0;
	int value = atoi(temp);
	delete [] temp;
	ignoreSpace();
	return value;
}

char * Analyse::getNextString()
{
	Expect('"');				
				
	char * temp = new char[strlen(p)+1];
	char *p2 = temp;
				
	while ( NONTERMINATEUR(*p) )
	{   
		switch ( *p )
		{
		case '\\':
			{
				switch ( * (p+1) )
				{
				case 'n':
					*(p2++)='\n';
					p+=2;
					break;
				case '"':
					*(p2++) = '"';
					p+=2;
					break;
				default:		  
					*(p2++) = *(p++);
				}
				break;
			}
		case '"':
			{
				*p2 = 0; // forget the last '"'
				p++;
				ignoreSpace();
				return temp;
			}
		default:
			*(p2++) = *(p++);
		}		
	}// end while
	throw new ErrAnalyse("Could not find the end of the string");
}
