/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#include "S_R.H"
#include "G_B_R_Pipe.H"  // To receive infos from clients
#include "S_Error.H"
#include "S_Users.H" // To send infos to users
// Different types of Posts
#include "Post_N.H"  
#include "Post_M.H"
#include "Post_L.H"
// To write the log
#include "Debug.H"

#include "S_wholist.H"
#include "consts.H"
#include "Special_Users.H"

/* See S_R.H for explanations about this class */

extern G_B_R_Pipe p;  // To receive infos from clients
extern S_Users U; // To send infos to users
extern G_wholist W;

S_R::S_R()
{
  // Init
  Debug(1,"S_R()\n");
  Room_N[0]=0; // room X    ( not used here )
  Room_N[1]=0; // Room Mail ( not used here ) 
  Room_L = new S_room_M(0); Room_L->Lit_Sauve(k_Read); // room X 
  Room_M = new S_room_M(1); Room_M->Lit_Sauve(k_Read); // Room Mail
  for (int n=2; n< MAX_ROOM; n++ ) // Normal rooms
    {
      Room_N[n] = new S_room_N(n); 
      Room_N[n]->Lit_Sauve(k_Read);
    } 
}

void S_R::flush(void)
{
  // Write infos on disk
  Debug(1,"S_R::flush()\n");

  for (int n =2;  n< MAX_ROOM; n++ ) 
      Room_N[n]->Lit_Sauve(k_Write);
  Room_M->Lit_Sauve(k_Write);
  Room_L->Lit_Sauve(k_Write);
}

S_R::~S_R()
{
  // Finish
  Debug(1,"~S_R()\n");
  flush();
  
  for ( int n =2;  n< MAX_ROOM; n++ ) 
      delete Room_N[n];
  delete Room_M;
  delete Room_L;
}

void S_R::Zap(unsigned short nom)
{
  LISTE a(MAX_ROOM);
  //  unsigned char a; // action 
  p.Lit(a);
  for (int i=0; i< a.size(); i++)
    {
      unsigned short room = a.DATA[i];
      try 
	{
	  // See if he is allowed to change or not
	  S_G_room * ZeRoom = VerifRoom(room, nom);
	  // Update pos
	  ZeRoom->SetPos(nom,65535);
	} catch (...) {}
    }
}

void S_R::Dispatch(unsigned short nom)
{
  unsigned char ch;
  p.Lit(ch);
  switch (ch)
    {
    case '!': Reload_Dat(nom); break;
    case ' ': Envoyer(nom); break;
    case 'k': Envoie_Liste(nom); break;
    case 'R': Bouge(nom); break;  
    case 'z': Zap(nom); break;
    case 'Z': Reload(nom); break;  // Receiving room information
    case 'd': Deplace(nom); break; //
    case 'D': DEL(nom); break;     // deleting a post
    case 'p': case 'P':            // receiving a post
      {
	int kick = U.AddRequest(nom);
	Receive(nom,ch == 'P'); 
        if ( kick ) 
	  {
	    Debug(3,"KICK of user %d ******\n",nom);
	    if ( U.Set(nom) )
	      {
		U.SendSX(user_sysop,S_flood_server);	   
		U.SendX(user_sysop,'k',"So long sucker\n",D_admin+D_untwit);
	      }
	  }
      }
      break; 
    case 'C': CopierPost(nom); break; // copying a post
    default:
      Debug(3,"S_R::Dispatch(nom=%d,char=%c):ERREUR PROTOCOLE !!\n",nom,ch);
    }
}



void S_R::CopierPost(unsigned short nom)
{
  unsigned short NumRoom,NumPost,NvelleRoom;
  p.Lit(NumRoom);
  p.Lit(NumPost);
  p.Lit(NvelleRoom);
  S_G_room * ZeRoom;
  ZeRoom = VerifRoom(NumRoom,nom);
  // We Get the post to copy
  Post * P = ZeRoom->GetPost(nom,NumPost,0);
  if ( ! P )
    return;
  // We see if the user can deal with this post
  if ( ! P->IsMine(nom) )
    return;
  // We see if the user is allowed to post in the destination room
  ZeRoom = VerifRoom(NvelleRoom,nom); 

  Post * CopyP = P->CopyPost ( );
  // Now that all the data is read ( pipe clean ), we see if this user
  // can post in this room
  // We add the post to the room
  CopyP->move_u = nom; 
  CopyP->move_r = NumRoom; 
  ZeRoom -> Poster(CopyP);
}

void S_R::Reload_Dat(unsigned short nom)
{
  unsigned short room;
  // Get number of room
  p.Lit(room); 
  
  // A EditRoom have modified the Room informations
  // Can the user touch this room ?
  S_G_room * ZeRoom = VerifRoom(room,nom); 
  Debug(0,"S_R::Reload(U=%d,R=%d)\n",nom,room);
#warning "Hummm, finir ici, c'est le box"
  //  ZeRoom->Reload_Dat();
}

void S_R::Reload(unsigned short num)
{
  // A EditRoom have modified the Room informations
  S_G_room * R = VerifRoom(num, 0);
  R->Reload();
}

void S_R::Receive(unsigned short nom, int reply)
{
  unsigned short room;
  Post * P;
  // Get number of room
  p.Lit(room);
  // Choose type of post to read
  switch ( room )
    {
    case 0 : 
      P= new Post_L(); 
      W.mem->Stat[N_ser_X]++;
      break;
    case 1 : P= new Post_M(); break;
    default: 
      P= new Post_N();
      W.mem->Stat[N_ser_Post]++; 
      break;
    }
  // Update stats
  //  if ( room ) W.mem->Stat[N_ser_Post]++; else W.mem->Stat[N_ser_X]++;
  // Read post
  P->Recoit(nom);
  // Now that all the data is read ( pipe clean ), we see if this user
  // can post in this room
  S_G_room * ZeRoom = VerifRoom(room,nom); 
  // Set infos if it is a reply post !!
  if ( reply )
    {
      P->ref_r = room;
      P->ref_n = ZeRoom->GetPos(nom);
      Post * Q = ZeRoom->GetPost(nom,P->ref_n,0);

      if ( Q )
	{
	  P->ref_s = Q->subject;
	  P->ref_u.Set(Q->username());
	}
    }
  // We add the post to the room
  ZeRoom -> Poster(P);
}

void S_R::Deplace(unsigned short nom)
{
  // The user "nom" wants to go to the pos "pos" in the room "room"
  unsigned short room, pos;
  p.Lit(room);
  p.Lit(pos);
  // See if he is allowed to change or not
  S_G_room * ZeRoom = VerifRoom(room, nom);
  // Update pos
  ZeRoom->SetPos(nom,pos);
}

void S_R::Envoyer(unsigned short nom)
{
  unsigned short numderoom, ZeRoom;
  // The user "nom" wants to get next post avaliable, 
  // starting at room "numderoom"

  p.Lit(numderoom);
  ZeRoom = numderoom;
  int TEST_BUG=0;

  if (!U.Set(nom)) return;

  S_G_room * ZeRoomPoil;
  Post * P;

  Debug(0,"S_R::Envoyer(Room=%d,User=%d)\n",numderoom, nom);

  // We first verify mail, a mail should always be the first post sent
  // if it exsits
  ZeRoomPoil = VerifRoom(1,nom); 

  if ( ! ( P = ZeRoomPoil->GetNext(nom) ) ) // No post for us in room Mail
    {
      // We check the first asked room
      ZeRoomPoil = VerifRoom(numderoom, nom);
      // We see if there is a post
      P = ZeRoomPoil->GetNext(nom);
      
      if ( (! P) && (! numderoom) )
	{
	  numderoom = 1; ZeRoom = 1;
	  ZeRoomPoil = VerifRoom(numderoom, nom);
	  P = ZeRoomPoil->GetNext(nom);
	}

      if (!P) do
	{
	  ZeRoom ++; TEST_BUG++;
	  if ( ZeRoom >= MAX_ROOM - 1 ) ZeRoom = 1; 
	  ZeRoomPoil = VerifRoom(ZeRoom, nom);
	  P = ZeRoomPoil->GetNext(nom);  
	  if ( TEST_BUG > MAX_ROOM +1 )
	    {
	      Debug(3,"ERROR !!: infinite while ?\n");
	      SePlacer(nom,2);
	      return;
	    }
	} while ( ZeRoom != numderoom && !P );
    } 

  if ( P  )
  {
    unsigned short der=ZeRoomPoil->GetLast(nom);
    U.Envoie((unsigned char)' ');
    U.Envoie(ZeRoomPoil->Num()); 
    U.Envoie(der);
    P->Envoie(nom);
  } else SePlacer(nom,2);
}

void S_R::SePlacer(unsigned short nom, unsigned short numderoom)
{
  S_G_room * ZeRoom = VerifRoom(numderoom, nom );

  Debug(0,"S_R::SePlacer(%d,%d)\n",nom,numderoom);
   
  if ( ! U.Set(nom) ) return; // Le SetUser sur ki est fait ...

  switch (  ZeRoom->State(nom) )
    {
    case '*': throw S_Error(S_no_room,nom);   break;
    case 'K': throw S_Error(S_kick_room,nom); break;
    case 'P': throw S_Error(S_no_acess_room ,nom);
    }
  
  U.Envoie((unsigned char)'R');
  U.Envoie(numderoom);
  unsigned short pos = ZeRoom->GetPos(nom), der =  ZeRoom->GetLast(nom);
  U.Envoie(der);
  U.Envoie(pos);
  U.flush();
}

void S_R::Bouge(unsigned short nom)
  // Un user demande a aller kekepart dans 1 room ...
{
  unsigned short room; // dit ou on est ( ou ou on veut aller )
  unsigned char direction;    // Type de "move"
  unsigned short ZeRoom;

  p.Lit(room);  
  p.Lit(direction);

  S_G_room * ZeRoomPoil = VerifRoom(room, nom );

  Debug(0,"S_R::Bouge(u=%d,r=%d,dir=%c)\n",nom,room,direction);

  switch ( direction )
  {
    case '+': case ')': // forward
      ZeRoom = room;
      do
      {
        ZeRoom ++;
	if ( ZeRoom >= MAX_ROOM ) ZeRoom = 0; 
	ZeRoomPoil = VerifRoom(ZeRoom, nom );
        char ST = ZeRoomPoil->State(nom);
        if ( ST=='N' || (ST=='F' && direction==')')  )
	  { SePlacer(nom,ZeRoom); return; }
      } while ( ZeRoom != room );
      SePlacer(nom,2);
    break;

    case '-': case '(': // backward
      ZeRoom = room;
      do
      {
        if ( ! ZeRoom ) ZeRoom = MAX_ROOM - 1;
        ZeRoom --;
	ZeRoomPoil = VerifRoom(ZeRoom, nom );
        char ST = ZeRoomPoil->State(nom);
        if ( ST=='N' || (ST=='F' && direction=='(')  )
	  { SePlacer(nom,ZeRoom); return; }
      } while ( ZeRoom != room );
      SePlacer(nom,2);
    break;

    case 'j': // jump
      if ( ! room ) // Room X_LOG, on va a la fin
	ZeRoomPoil->SetPos(nom,ZeRoomPoil->GetLast(nom));
      SePlacer(nom,room);
    break;
  default:
    Debug(3,"Protocol error S_R_Bouge !!\n");
  }
}

void S_R::Envoie_Liste(unsigned short nom)
{
  if ( ! U.Set(nom) ) return;

  unsigned short pos;

  U.Envoie((unsigned char)'k');

  unsigned char ch;
  for (unsigned short i =0; i<MAX_ROOM;i++ )
  {
    try {
      S_G_room * ZeRoomPoil = VerifRoom(i, nom );
      ch = ZeRoomPoil->State(nom);
      if ( (ch != '*') && (ch != 'P') )
	{
	  pos = ZeRoomPoil->GetPos(nom);
	  U.Envoie(i);
	  U.Envoie(ch);
	  U.Envoie(pos);
	}
    }
    catch ( S_Error S )
      {
	Debug(2,"S_ERROR '%d' on '%d'\n",S.koa(),S.ki());
      }
  }

  pos=65535;
  U.Envoie(pos);
  U.flush();
}

S_G_room * S_R::VerifRoom(unsigned short room, unsigned short nom)
{
  if ( room > MAX_ROOM -1 )
    { 
      throw S_Error(S_no_room,nom);
    }
  if ( room == 0 )
    {
      if ( nom > MAX_USERS - 1 ) throw S_Error(S_no_user,0);
      return Room_L;
    }
  if ( room == 1 )
    {
      if ( nom > MAX_USERS - 1 ) throw S_Error(S_no_user,0);
      return Room_M;
    }
  if ( ! Room_N[room]  ) throw S_Error(S_no_room,nom);
  return Room_N[room];
}


void S_R::DEL(unsigned short nom)
{
  unsigned short room, post;
  p.Lit(room);
  p.Lit(post);
  if ( ! U.Set(nom) ) return; // Apres la lecture des infos ( protocole )
  S_G_room * ZeRoom = VerifRoom(room, nom);
  ZeRoom ->Deleter(post,nom);
}
