/*
   Copyright (C) 1992 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
   This program have been made by: Sylvain DURAND , tilde@club-internet.fr
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#ifndef __SERVER_H_
#define __SERVER_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "Socket.H"

#include "G_Path.H"
#include "Con_Wholist.H"
#include "C_Pipe.H"
#include "C_User_C.H"

// Socket exit status
#define SLSOCK_CREATE		-101
#define SLSOCK_BIND		-102
#define SLSOCK_GETSOCKNAME	-103
#define SLSOCK_LISTEN		-104
#define SCPIPE_CREATE		-105
#define PARENT_SELECT		-106
#define SLSOCK_ACCEPT		-107
#define PARENT_FORK		-108
#define CHILD_SELECT		-109

// Protocol exit status
#define CL_CANNOT_LOG		-201

// Normal exit status
#define CL_ASK_LOGOUT		-302

//Configuration
#define SERV_PORT               "2000"		// Port du serveur par defaut
#define VERSION                 "0.1.20"

#define MAX_USER		10



class Deconnection {};

class server
{
private:
	  int nb_user;		//nombre utilisateur deja connecte
	  int listen_socket;	// Socket d'ecoute du serveur
	  int client_socket;	// Socket de communication avec le client
	  int comm_pipe [2];	// serv_comm_pipe [0] is listened by parent process
				// serv_comm_pipe [1] to write to parent (admin)
	  struct sockaddr_in server_name;	// Connection et adresse internet du serveur
	  struct sockaddr_in client_name;	// Connection et adresse internet du client
	  unsigned int server_len;		// Sizeof (server_socket)
	  unsigned int client_len;		// Sizeof (client_socket)
	  
public:
	  server (char *);
	  int run (void);
	  
private:
	  void server_listen_create (void);
	  void server_name_identify (char *);
	  void server_bind (void);
	  void server_getsockname (void);
	  void server_listen_mode (void);
	  void server_comm_create (void);
	  void signal_update (const int);
	  void init_server_set (int *, fd_set *);
	  void server_select (int, int *, fd_set *);
	  void test_nbselret (int, int *);
	  void server_wait (int, fd_set *);
	  void server_admin_message (void);
	  void server_accept_test (int *);
	  void server_client_accept (void);
	  void server_child_connect (int *, int *);
	  void server_incoming_connection (fd_set *, int *, int *);
	  int end_of_run (int);
	  int verify_pass (char *, char *);
	  char verify_login (char *, char *);
	  void zero_fd (int *, fd_set *);
	  void set_fd (int *, int, fd_set *);
};
#endif
