/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
/*
   This program have been made by: Sylvain DURAND , tilde@club-internet.fr
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#include "Server.H"

G_Path G (0);			// Process ne s'arretant jamais

Con_Wholist W;			// Te permet mater mem partagee

C_Pipe P;


void
terminate (void)
{
  printf ("Ze termine\n");
  exit (0);
}

void
server_child_termination (int sign)
{
  if (sign == SIGCHLD)
    {
      signal (SIGCHLD, server_child_termination);	// relaunch the signal

      wait (NULL);
    }
}

void server::
server_listen_create (void)
{
/***********************************************/
/* Creation du socket d'ecoute 'listen_socket' */
/* Pour la connection du serveur sur le server */
/***********************************************/

  if ((listen_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
      perror ("Fatal error: Unable to open server's listen socket");
      exit (SLSOCK_CREATE);
    }
}

void server::
server_name_identify (char *serv_port)
{
/***********************************************
* On remplit la structure bbs_server avec les  *
* caracteristiques du listen_socket:           *
* 1) sin_family : type de connection           *
* 2) sin_port : port                           *
* 3) sin_addr : adresse internet du bbs (IP)   *
***********************************************/

  server_name.sin_family = AF_INET;
  server_name.sin_port = htons (atoi (serv_port));
  server_name.sin_addr.s_addr = INADDR_ANY;	// -> getsockname

}

void server::
server_bind (void)
{
/**********************************************************
* Tentative de bind du listen_socket avec le server_name  *
***********************************************************/

  if ((bind (listen_socket, (struct sockaddr *) &server_name, sizeof (server_name))) < 0)
    {
      close (listen_socket);
      perror ("Fatal error: Unable to bind server's listen socket");
      exit (SLSOCK_BIND);
    }
}

void server::
server_getsockname (void)
{
/********************************************************************
* Ici on obtient le nom du serveur et on le stocke dans server_name *
*********************************************************************/

  server_len = sizeof (server_name);

  if ((getsockname (listen_socket, (struct sockaddr *) &server_name, &server_len)) < 0)
    {
      close (listen_socket);
      perror ("Fatal error: Unable to get server's listen socket name");
      exit (SLSOCK_GETSOCKNAME);
    }

}

void server::
server_listen_mode (void)
{
/*******************************************
* On passe le listen_socket en mode listen *
********************************************/

  if ((listen (listen_socket, 5)) < 0)
    {
      close (listen_socket);
      perror ("Fatal error: Unable to listen to server's listen socket");
      exit (SLSOCK_LISTEN);
    }
}

void server::
server_comm_create (void)
{
/********************************
* Create the communication pipe *
* to administrate the server    *
*********************************/

  if ((pipe (comm_pipe)) < 0)
    {
      close (listen_socket);
      perror ("Fatal error: Unable to create server's communication pipe");
      exit (SCPIPE_CREATE);
    }
}

server::server (char *serv_port = SERV_PORT)
{
  printf ("Pipo Server v%s build on %s at %s\n", VERSION, __DATE__, __TIME__);
  printf ("Max user: %d\n", MAX_USER);

  nb_user = 0;

  server_listen_create ();
  server_name_identify (serv_port);
  server_bind ();
  server_getsockname ();

#ifdef DEBUG
  printf ("DEBUG : ntohs (server_name.sin_port) : ");
  printf ("%d\n", ntohs (server_name.sin_port));
  printf ("DEBUG : ntohl (server_name.sin_addr.s_addr) : ");
  printf ("%x\n", ntohl (server_name.sin_addr.s_addr));
#endif

  server_listen_mode ();
  server_comm_create ();
}

int server::
verify_pass (char *login_name, char *login_pass)
{
  FILE * f;
  char PASSWD[MAX_PASSWD+1];
  char * temp=G.GetUserFile(login_name,"pswd");
  
  f=fopen(temp,"r");
  delete [] temp;

  if (! f )
    return 0;

  fgets(PASSWD,MAX_PASSWD,f);
  PASSWD[strlen(PASSWD)-1]=0;
  fclose(f);

#ifdef DEBUG
  printf("=>Password:%s\n",PASSWD);
#endif
  int ok = ! strcmp(PASSWD,login_pass);

  return ok;
}

char server::
verify_login (char *login_name, char *login_pass)	//Mettre les vrais testes !!!
 {
  if (nb_user > MAX_USER)
    return 'M';

  if (strcmp (login_name, "Off") == 0)
    return 'S';
  /*
  if (strcmp (login_name, "poil") == 0)
    return 'L';
  */
  if (verify_pass(login_name, login_pass))
    return 'O';

  return 'P';
}

void server::
signal_update (const int Ze_signal)
{
/********************************************************
* On prevoit la reception de signaux, comme  SIGCHLD    *
* prevenant du deces des child process qui se terminent *
*********************************************************/

  if (Ze_signal == SIGCHLD)
    signal (SIGCHLD, server_child_termination);
}

void server::
init_server_set (int *maxfdinset, fd_set * fd_serv_set)
{
/*********************************/
  /* Initialize the set with no fd */
/*********************************/

  zero_fd (maxfdinset, fd_serv_set);

/****************************************************/
  /* Add the comm server's pipe and socket to the set */
/****************************************************/

  set_fd (maxfdinset, listen_socket, fd_serv_set);
  set_fd (maxfdinset, comm_pipe[0], fd_serv_set);
}

void server::
server_select (int maxfdinset, int *nbselret, fd_set * fd_serv_set)
{
/***********************************************/
  /* Waiting for an incoming message on the pipe */
  /* or on the socket with select                */
  /* N.B: HP_UX needs her select having 2nd arg  */
  /* to be int* type                             */
/***********************************************/

#ifdef HP_UX
  *nbselret = select (maxfdinset, (int *) fd_serv_set, NULL, NULL, 0);
#else
  *nbselret = select (maxfdinset, fd_serv_set, NULL, NULL, 0);
#endif
}

void server::
test_nbselret (int nbselret, int *relaunch_accept)
{
  if (nbselret <= 0)
    {
      if (errno != EINTR)	// La c'est pas normal

	{
	  close (listen_socket);
	  perror ("Fatal error: Unable to accept a connection from client");
	  exit (PARENT_SELECT);
	}
      else			// La c'est normal: un fils est mort

	{
	  printf ("Connection status: correctly closed\n");
	  *relaunch_accept = 1;
	  nb_user--;
	}
    }
}

void server::
server_wait (int maxfdinset, fd_set * fd_serv_set)
{
  int relaunch_accept = 0;
  int nbselret;

  do
    {
      server_select (maxfdinset, &nbselret, fd_serv_set);
      relaunch_accept = 0;
      test_nbselret (nbselret, &relaunch_accept);
    }
  while (relaunch_accept);
}

void server::
server_admin_message (void)
{
  printf ("Incoming child message!!!\n");
}

void server::
server_accept_test (int *relaunch_accept)
{
  if ((client_socket = accept (listen_socket, (struct sockaddr *) &client_name, &client_len)) < 0)
    {
      if (errno != EINTR)	// La c'est pas normal

	{
	  close (listen_socket);
	  perror ("Fatal error: Unable to accept a connection from client");
	  exit (SLSOCK_ACCEPT);
	}
      else			// La c'est normal: un fils est mort

	{
	  printf ("Connection status: correctly closed\n");
	  *relaunch_accept = 1;
	  nb_user--;
	}
    }
}

void server::
server_client_accept (void)
{
  int relaunch_accept = 0;

/**************************************/
  /* On attend une connection du client */
  /* a l'aide de la primitive accept    */
/**************************************/

  do
    {
      relaunch_accept = 0;
      server_accept_test (&relaunch_accept);
    }
  while (relaunch_accept);
}

void server::
server_child_connect (int *logout, int *pid)
{
  client_len = sizeof (client_name);	//il faut recuperer le hot !

  server_client_accept ();

  nb_user++;

  printf ("nombre utilisateurs: %d\n", nb_user);




/********************************************/
  /* Il est tant de forker le serveur :       */
  /* Le pere reste celui qui fait les accepts */
  /* Le fils gere la nouvelle connection      */
/********************************************/

  if ((*pid = fork ()) < 0)
    {
      close (client_socket);
      close (listen_socket);
      perror ("Fatal error: Fork Failure\n");
      exit (PARENT_FORK);
    }

  if (*pid != 0)		// Parent process

    {
/***********************************************/
      /* On ferme le socket qu'on vient de connecter */
      /* Et on reprend l'attente d'accept            */
/***********************************************/

      close (client_socket);

    }				// End of parent

  else				// Child process

    {
      int result_log;
      char *ver;
      char *login_name;
      char *login_pass;


      ver = new char[VER_LEN];
      login_name = new char[LG_NOM_USER];
      login_pass = new char[MAX_PASSWD];

      /***********************************************/
      /* On ferme le socket listen du serveur car    */
      /* l'accept est gere par le pere               */
      /* We also close the comm_pipe [0] which reads */
      /* commands and is only used by parent         */
      /***********************************************/

      close (listen_socket);
      close (comm_pipe[0]);

      /************************************************/
      /* Creation of the conection to the pipe server */
      /* Mon boulot :                                 */
      /************************************************/

      Socket ToClient (client_socket);

      ver[0] = '\0';

      result_log = ToClient.GetVer (&ver);
      printf ("Protocol version= %s\n", ver);

      do
	{

	  login_name[0] = '\0';
	  login_pass[0] = '\0';
	  result_log = ToClient.GetLogin (&login_name, &login_pass);
	  //On retaille le tout a la bonne longueur
	  login_name[LG_NOM_USER] = '\0';
	  login_pass[MAX_PASSWD] = '\0';

	  if (result_log < 0)
	    {
	      ToClient.SendAck ('L', "protocol error");
	      printf ("Unable to understand login message.\n");
	      close (listen_socket);
	      perror ("Client unable to give log and pass");
	      exit (CL_CANNOT_LOG);
	    }
	  else
	    {
	      switch (verify_login (login_name, login_pass))
		{
		case 'M':
		  {
		    ToClient.SendAck ('L', "max user");
		    printf ("Maximum user (%d) reached on socket server.\n", MAX_USER);
		    printf ("Try later ...\n");
		    close (listen_socket);
		    exit (CL_CANNOT_LOG);
		  }
		  break;
		case 'O':
		  {
		    ToClient.SendAck ('L', "ok");
		    printf ("Accept log %s.\n", login_name);
		  }
		  break;
		case 'S':
		  {
		    ToClient.SendAck ('L', "stop execution");
		    printf ("Closing connection (%s user).\n", login_name);
		    close (listen_socket);
		    exit (CL_ASK_LOGOUT);
		  }
		  break;
		case 'P':
		  {
		    ToClient.SendAck ('L', "wrong password");
		    printf ("Wrong pass for %s\n", login_name);
		    result_log = -1;
		  }
		  break;
		case 'L':
		  {
		    ToClient.SendAck ('L', "bad user");
		    printf ("User %s doesn't exist\n", login_name);
		    result_log = -1;
		  }
		  break;
		}
	    }
	}
      while (result_log < 0);

      C_User_C ToServer (login_name);	// login_name contient le nom du client

      char *tmp;
      tmp = new char[NUMLEN];

      printf("->num: %d\n",P.numdeuser); // Affcihe le numero du username
      
      sprintf(tmp,"%d",P.numdeuser);
      ToClient.SendAck('N', tmp); // Envoie le numero du username au client concerne

      delete [] tmp;

      /*****************************************************/
      /* On indique au socket l'adresse du pipe du serveur */
      /*****************************************************/

      ToClient.Set (&ToServer);

      /****************************************************/
      /* Boucle principale de reception/envoi de messages */
      /****************************************************/

      while (!(*logout))
	{
	  try
	  {

	    /***********************************************/
	    /* Add the client's pipe and socket to the set */
	    /***********************************************/

	    ToServer.ResetSelect();
	    ToServer.AddSelect(ToServer.f_r);
	    ToServer.AddSelect(client_socket);

	    /***********************************************/
	    /* Waiting for an incoming message on the pipe */
	    /* or on the socket with select                */
	    /* N.B: HP_UX needs her select having 2nd arg  */
	    /* to be int* type                             */
	    /***********************************************/


	    if (ToServer.Select(0) <= 0) // 0 = pas de timeout
	      {
		perror ("Fatal error: Select failure\n");
		exit (CHILD_SELECT);
	      }

	    if (ToServer.IsSelect(client_socket))
			 //   (client_socket, &fdset)	// Receiving a message from distant client
	       {
		W.IDLE ();	// User is alive and not idle
  
		ToClient.Gere ();
	      }

	    if (ToServer.IsSelect(ToServer.f_r) && ToServer.InitRead() )
	      // Receiving a message from bbs
	       {
		unsigned char ch;

		ToServer.Lit (ch);
		switch (ch)
		  {
		  case 'X':
		    {
		      printf ("Receiving X from bbs server\n");
		      Un_X x;

		      //Faire un switch afin de recuperer les message K
		      //Au moins pas de bidouille possible par les sources
		      //du client....

		      ToServer.Get_X (x);
		      printf ("Message '%s' de %d.\n", x.buf, x.ki);
		      ToClient.SendX (x.type, x.buf, x.ki);
		    }
		    break;
		  case ' ':
		    {
		      printf ("Le serveur vient de m'envoyer un post :)\n");
		    }
		    break;
		  case 'N':
		    {
		      printf ("Le serveur me demande de relire les infos\n");
		      ToServer.Lit_Sauve (k_Read);
		      printf ("J'envoie un pipo X a AA\n");
		    }
		    break;
		  case 'a':
		  case 'A':
		    {
		      Un_Ack ack;
		      char *tmp;
		      tmp = new char[NUMLEN];

		      ToServer.Get_Ack (ch == 'a', ack);
		      printf ("Receiving an acknowledgment of X de %d\n", ack.ki);

		      sprintf(tmp,"%d",ack.ki); //Refaire ca c LAID !!!
		      ToClient.SendAck ('X', tmp);
		      
		      delete [] tmp;
		    }
		    break;
		  case 'R':
		    {
		      printf ("Le serveur me signale que je change de roume\n");
		      Un_Roume r;

		      ToServer.Get_Room (r);
		      printf ("AA, au turf\n");
		    }
		    break;
		  case 'S':
		  case 'K':
		    {
		      printf ("Receiving a Connect/Deconnect signal from server\n");
		      // Hello_Baille a (ch =='S');
		      if (ch == 'S')
			{
			  printf ("Le serveur dit hello\n");
			  W.Set (&ToServer);
			  W.Update(); //Met a jour la wholist (flag,doing pour le user)

			  Un_Roume r;
			  printf ("Num Room %d\n",r.numroom);
			}
		      else
			{
			  printf ("le serveur dit baille\n");
			  throw Deconnection ();
			}
		    }
		    break;
		  case 'h':
		    {
		      Un_log l;
		      char *tmp;

		      tmp = new char[NUMLEN];

		      ToServer.Get_Log (ch == 'h', l);
		      printf ("%d se loge (A FAIRE)\n", l.ki);

		      sprintf(tmp,"%d",l.ki);
		      ToClient.SendAck('C',tmp);

		      delete [] tmp;
		    }
		    break;
		  case 'b':
		    {
		      Un_log l;
		      char *tmp;

		      tmp = new char[NUMLEN];

		      ToServer.Get_Log (ch == 'b', l);
		      printf ("%d se deloge (A FAIRE)\n", l.ki);
		      
		      sprintf(tmp,"%d",l.ki);
		      ToClient.SendAck('D',tmp);
		      
		      delete [] tmp;
		    }
		    break;
		  case 'k':
		    {
		      printf ("Le serveur m'envoie la liste des roumes\n");
		      RListe r;

		      ToServer.Get_RListe (r);
		      printf ("AA, o turf !\n");
		    }
		    break;
		  case 'T':
		    {
		      printf ("Le serveur m'envoie une info sur le tag :)\n");
		      Un_Tag t;

		      ToServer.Get_Tag (t); //Il faut envoyer un Ack Tag
		      //Il faut faire un switch.....
		      //pour les differentes operations
		    }
		    break;
		  default:
		    printf ("error protocole: %c(%d)\n", ch, ch);
		  }
	      }
	  }			// Fin de Try

	  catch (Deconnection)
	  {
	    printf ("Server asking for deconnection\n");
	    *logout = 1;
	  }

	  // Gere les autres catch
	  // PipeProbs, PlusPersonne, ....

	}			// Fin while (!logout) du fils

      delete[]login_name;
      delete[]login_pass;
    }				// Fin du process du fils

}

void server::
server_incoming_connection (fd_set * fd_serv_set, int *logout, int *pid)
{
  if FD_ISSET
    (comm_pipe[0], fd_serv_set)	// Receiving an administration message from child
       server_admin_message ();
  if FD_ISSET
    (listen_socket, fd_serv_set)	// Receiving a login attempt from distant client
       server_child_connect (logout, pid);
}

int server::
end_of_run (int pid)
{
  /**********************************************/
  /* La tache de ce serveur est finie.          */
  /* Et la poussiere retourne a la poussiere... */
  /* Si c'est un fils, on exit pour eviter      */
  /* qu'il poursuive son execution.             */
  /**********************************************/

  if (pid == 0)			// C'est le fils :

    {
      printf ("Child server correctly logged out\n");
      nb_user--;
      exit (0);			// On exite le programme pour stopper l'execution du process.

    }
  else				// C'est le pere :

    {
      printf ("Server's run correctly terminated\n");
      return 0;			// On retourne 0 et on continue l'execution.

    }
}

int server::
run (void)
{
  int pid = 0;			// Contient le pid du dernier fils forke

  int logout = 0;		// Condition d'arret du serveur

  int endval;

  signal_update (SIGCHLD);

  while (!logout)
    {
      fd_set fd_serv_set;	// File descriptor set for select

      int maxfdinset;		// Max value +1 fd within a set

      init_server_set (&maxfdinset, &fd_serv_set);
      server_wait (maxfdinset, &fd_serv_set);
      server_incoming_connection (&fd_serv_set, &logout, &pid);
    }

  endval = end_of_run (pid);

  return endval;
}

void server::
zero_fd (int *max, fd_set * fdset)
{
  FD_ZERO (fdset);
  *max = 0;
}

void server::
set_fd (int *max, int fd, fd_set * fdset)
{
  FD_SET (fd, fdset);
  if ((fd + 1) > *max)
    *max = fd + 1;
}

int
main (void)
{
  server serv;

  return serv.run ();
}
