/* ***************************************************************** *
 * Atlantis BBS - CGI Colors Module                                  *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.5 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Mon 10 Aug 1998 *
 * Modification date . . . . . . . . . . . . . . . . Fri 28 May 1992 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *   Cyril GIRON <cgiron@writeme.com>                                *
 *   Gregory VANDENBROUCK <vdb@pipo.com>                             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <errno.h>


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "log_error.h"


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char program[]   = "CGI Colors Module";
static char null[]      = "";
static char backslash[] = "\\";
static char ent_amp[]   = "&amp;";
static char ent_lt[]    = "&lt;";
static char ent_gt[]    = "&gt;";

/* ANSI Colors (ISO 6429) ****************************************** */
char  fg_dark[]         = "<FONT Color=\"#000000\">";
char  fg_red[]          = "<FONT Color=\"#FF0000\">";
char  fg_green[]        = "<FONT Color=\"#00FF00\">";
char  fg_blue[]         = "<FONT Color=\"#0000FF\">";
char  fg_yellow[]       = "<FONT Color=\"#FFFF00\">";
char  fg_magenta[]      = "<FONT Color=\"#FF00FF\">";
char  fg_cyan[]         = "<FONT Color=\"#00FFFF\">";
char  fg_white[]        = "<FONT Color=\"#FFFFFF\">";
char  fg_end[]          = "</FONT>";

char  bg_dark[]         = "<FONT STYLE=\"background: black\">";
char  bg_red[]          = "<FONT STYLE=\"background: red\">";
char  bg_green[]        = "<FONT STYLE=\"background: green\">";
char  bg_blue[]         = "<FONT STYLE=\"background: blue\">";
char  bg_yellow[]       = "<FONT STYLE=\"background: yellow\">";
char  bg_magenta[]      = "<FONT STYLE=\"background: magenta\">";
char  bg_cyan[]         = "<FONT STYLE=\"background: cyan\">";
char  bg_white[]        = "<FONT STYLE=\"background: white\">";
char  bg_end[]          = "</FONT>";

/* Particular Attributes ******************************************* */
char  at_blink[]        = "<Blink>";
char  at_blink_end[]    = "</Blink>";

char  at_bold[]         = "<B>";
char  at_bold_end[]     = "</B>";

char  at_underline[]    = "<U>";
char  at_underline_end[]= "</U>";

char  at_reset[]        = "</U></B></FONT></Blink>";



/* ***************************************************************** *
 * bbs2html()                                                        *
 * --------                                                          *
 * description                                                       *
 *   This function translates the bbs sequences used to code the     *
 *   ANSI colors into HTML sequences.                                *
 *                                                                   *
 * explanation                                                       *
 *   In order to avoid many new calls, we do two passes.             *
 *   During the first pass, we determine the amount of supplemental  *
 *   memory that will be necessary to construct the HTML tags we'll  *
 *   generate in the second pass.                                    *
 *                                                                   *
 * note                                                              *
 *   Canonic use this fonction:                                      *
 *                                                                   *
 *       char * str, html;                                           *
 *       html = bbs2html(str);                                       *
 *       // do some stuff...                                         *
 *       delete html;                                                *
 *                                                                   *
 *   In order to simplify the use, we always duplicate the string,   *
 *   even if no modification was made.                               *
 *                                                                   *
 * ***************************************************************** */
char * bbs2html(char * str) {
    char *  html     = NULL;
    char *  tag      = NULL;
    int     str_len  = strlen(str);
    int     html_len = 0;
    int     needed   = 0;
    int     i        = 0;
    int     k        = 0;
    
    
    // -- First pass --
    // We determine the amount of memory that will
    // be necessary to construct the HTML tags. 
    for(i=0; i<str_len; i++) {
        if(str[i] == '\\') {
            // Special character. 
            i++;
            switch(str[i]) {
                case '&': 
                    needed += strlen(ent_amp);
                    break;
                
                case '<': 
                    needed += strlen(ent_lt);
                    break;
                
                case '>': 
                    needed += strlen(ent_gt);
                    break;
                
                case 'a': // Reset attribute
                    needed += strlen(at_reset);
                    break;
                
                case 'b': // Foreground blue
                    needed += strlen(fg_blue);
                    break;
                
                case 'B': // Background blue
                    needed += strlen(bg_blue);
                    break;
                
                case 'c': // Foreground cyan
                    needed += strlen(fg_cyan);
                    break;
                
                case 'C': // Background cyan
                    needed += strlen(bg_cyan);
                    break;
                
                case 'd': // Foreground dark
                    needed += strlen(fg_dark);
                    break;
                
                case 'D': // Background dark
                    needed += strlen(bg_dark);
                    break;
                
                case 'e': // Blink attribute
                    needed += strlen(at_blink);
                    break;
                
                case 'f': // Bold attribute
                    needed += strlen(at_bold);
                    break;
                
                case 'g': // Foreground green
                    needed += strlen(fg_dark);
                    break;
                
                case 'G': // Background green
                    needed += strlen(bg_dark);
                    break;
                
                case 'h': // Hidden attribute
                    // well.. another time... :-)
                    break;
                
                case 'm': // Foreground magenta
                    needed += strlen(fg_magenta);
                    break;
                
                case 'M': // Background magenta
                    needed += strlen(bg_magenta);
                    break;
                
                case 'p': // Foreground pink
                    needed += strlen(fg_magenta);
                    break;
                
                case 'P': // Background pink
                    needed += strlen(bg_magenta);
                    break;
                
                case 'r': // Foreground red
                    needed += strlen(fg_red);
                    break;
                
                case 'R': // Background red
                    needed += strlen(bg_red);
                    break;
                
                case 'u': // Underline attribute
                    needed += strlen(at_underline);
                    break;
                
                case 'w': // Foreground white
                    needed += strlen(fg_white);
                    break;
                
                case 'W': // Background white
                    needed += strlen(bg_white);
                    break;
                
                case 'y': // Foreground yellow
                    needed += strlen(fg_yellow);
                    break;
                
                case 'Y': // Background yellow
                    needed += strlen(bg_yellow);
                    break;
                
                default: // Unknown special character
                    break;
            }
        }
    }
    
    
    // Now, we exactly know how much memory we need. 
    
    if(needed == 0) {
        // No memory needed ?
        // That means there's no special things. So we just
        // return a copy of the given string unchanged. 
        html = new char[strlen(str) + 1];
        strcpy(html, str);
        return html;
    }
    
    html_len = str_len + needed + 1;
    
    html = new char[html_len];
    
    if(html == NULL) {
        fprintf(stderr, "%s: new[%d] failed", program, html_len);
        return NULL;
    }
    
    for(i=0; i<html_len; i++) {
        html[i] = 0;
    }
    
    // If we are here, we can asssume that html[] is correctly allocated. 
    
    // -- Second pass --
    // We can now construct the HTML tags. 
    for(i=0; i<str_len; i++, k++) {
        
        // CR are replaced with NUL
        if(str[i] == '\015') {
            str[i] = 0;
        }
        
        tag = null;
        if(str[i] == '\\') {
            // Special character. 
            i++;
            switch(str[i]) {
                case '\\': 
                    tag = backslash;
                    break;
                
                case '&': 
                    tag = ent_amp;
                    break;
                
                case '<': 
                    tag = ent_lt;
                    break;
                
                case '>': 
                    tag = ent_gt;
                    break;
                
                case 'a': // Reset attribute
                    tag = at_reset;
                    break;
                
                case 'b': // Foreground blue
                    tag = fg_blue;
                    break;
                
                case 'B': // Background blue
                    tag = bg_blue;
                    break;
                
                case 'c': // Foreground cyan
                    tag = fg_cyan;
                    break;
                
                case 'C': // Background cyan
                    tag = bg_cyan;
                    break;
                
                case 'd': // Foreground dark
                    tag = fg_dark;
                    break;
                
                case 'D': // Background dark
                    tag = bg_dark;
                    break;
                
                case 'e': // Blink attribute
                    tag = at_blink;
                    break;
                
                case 'f': // Bold attribute
                    tag = at_bold;
                    break;
                
                case 'g': // Foreground green
                    tag = fg_green;
                    break;
                
                case 'G': // Background green
                    tag = bg_green;
                    break;
                
                case 'h': // Hidden attribute
                    // well.. another time... :-)
                    break;
                
                case 'm': // Foreground magenta
                    tag = fg_magenta;
                    break;
                
                case 'M': // Background magenta
                    tag = bg_magenta;
                    break;
                
                case 'p': // Foreground magenta
                    tag = fg_magenta;
                    break;
                
                case 'P': // Background magenta
                    tag = bg_magenta;
                    break;
                
                case 'r': // Foregroung red
                    tag = fg_red;
                    break;
                
                case 'R': // Backgroung red
                    tag = bg_red;
                    break;
                
                case 'u': // Underline attribute
                    tag = at_underline;
                    break;
                
                case 'w': // Foreground white
                    tag = fg_white;
                    break;
                
                case 'W': // background white
                    tag = bg_white;
                    break;
                
                case 'y': // Foreground yellow
                    tag = fg_yellow;
                    break;
                
                case 'Y': // Background yellow
                    tag = bg_yellow;
                    break;
                
                default: // Unknown special character
                    break;
            }
            
            // If we're here, that's mean we have to insert an HTML tag. 
            strcat(html, tag);
            k += strlen(tag) - 1;
        
        } else {
            // Regular character.
            html[k] = str[i];
        }
    }
    
    html[k] = 0;
    
    return html;
}

