/* ***************************************************************** *
 * Atlantis BBS - CGI Room Info                                      *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.2 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Wed 02 Dec 1998 *
 * Modification date . . . . . . . . . . . . . . . . Fri 28 May 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>


/* ***************************************************************** *
 * Unix includes                                                     *
 * ***************************************************************** */
#include <sys/types.h>
#include <sys/stat.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "G_Path.H"
#include "consts.H"
#include "MAX.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_room.h"
#include "log_error.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF         255


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char     program[]           = "CGI Room Info";
static char     version[]           = "1.0.2";

static char     Room_name[]         = "nom =";
static char     Flags_attribute[]   = "flags =";
static char     Positions_pointers[]= "Pos = ";

static G_Path   G(60)               ; // max idle: 60 sec


/* ***************************************************************** *
 * get_room_info()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This function gets information about the specified room. The    *
 *   room number is passed via the num argument. name must be a      *
 *   pointer to a buffer of LG_NOM_ROOM+1 characters (defined in     *
 *   MAX.H). The room attributes are stored in the flags parameter.  *
 *   The function returns 1 on success and zero on failure.          *
 *                                                                   *
 * ***************************************************************** */
int get_room_info(int num, char * name, int * flags) {
    FILE *  file = NULL;
    char *  path = NULL;
    char    buf[BUF+1];
    int     needed = 1;
    int     i = 0;
    
    path = G.GetRoomFile("Infos", num);
    
    file = fopen(path, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
                "%s: can't open '%s': error %d, %s\n", 
                program, path, errno, strerror(errno)
        );
        return 0;
    }
    
    for(i=0; i<LG_NOM_ROOM+1; i++) {
        name[i] = 0;
    }
    
    while(needed) {
        fgets(buf, BUF, file);
        
        if(strstr(buf, Room_name) != NULL) {
            strcpy(name, buf + strlen(Room_name));
            name[strlen(name) - 1] = 0;
            continue;
        }
        
        if(strstr(buf, Flags_attribute) != NULL) {
            *flags = atoi(buf + strlen(Flags_attribute));
            continue;
        }
        
        if(strstr(buf, Positions_pointers) != NULL) {
            needed = 0;
            break;
        }
    }
    
    return 1;
}


/* ***************************************************************** *
 * get_room_description()                                            *
 * --------------------                                              *
 * description                                                       *
 *   This function gets the description of the specified room in     *
 *   the specified language.                                         *
 *   Passing the star character (*) acts as a joker and returns the  *
 *   first available romm description.                               *
 *                                                                   *
 * ***************************************************************** */
char * get_room_description(int num, char * language) {
    struct stat info;
    char *  s = NULL;
    int     k = 0;
    
    if(strcmp(language, "fr") == 0) return G.GetRoomInfo(num, L_fr);
    if(strcmp(language, "en") == 0) return G.GetRoomInfo(num, L_uk);
    if(strcmp(language, "uk") == 0) return G.GetRoomInfo(num, L_uk);
    if(strcmp(language, "sp") == 0) return G.GetRoomInfo(num, L_sp);
    if(strcmp(language, "it") == 0) return G.GetRoomInfo(num, L_it);
    if(strcmp(language, "no") == 0) return G.GetRoomInfo(num, L_no);
    //if(strcmp(language, "de") == 0) return G.GetRoomInfo(num, L_de);
    
    if(strcmp(language, "*") == 0) {
        for(k=0; k<16; k++) {
            s = G.GetRoomInfo(num, k);
            if(s != NULL && stat(s, &info) == 0) return s;
        }
    }
    
    return NULL;
}

