/* ***************************************************************** *
 * Atlantis BBS - Room Info CGI                                      *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.0 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Wed 25 Nov 1998 *
 * Modification date . . . . . . . . . . . . . . . . Mon 22 Mar 1998 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "consts.H"
#include "MAX.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_colors.h"
#include "CGI_parser.h"
#include "CGI_HTML.h"
#include "CGI_room.h"
#include "log_error.h"


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char program[]           = "Room Info CGI";
static char version[]           = "1.0.0a6";

static char Language[]          = "lang";
static char Room_Name[]         = "id";
static char Default_Language[]  = "en";

static char*lang                = NULL;



/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(void) {
    HASH *  data = NULL;
    FILE *  file = NULL;
    char *  path = NULL;
    char    str[1025];
    char *  html = NULL;
    char    name[LG_NOM_ROOM+1];
    int     room = 0;
    int     flags = 0;
    int     n = 0;
    
    open_log_file();
    setbuf(stdout, NULL);  // Unbuffer stdout.
    setbuf(stderr, NULL);  // Unbuffer stderr.
    
    data = expect(5);
    parse(data);
    
    lang = get_value(data, Language);
    
    if(lang == NULL) {
        lang = Default_Language;
    }
    
    room = (get_value(data, Room_Name) != NULL) ? atoi(get_value(data, Room_Name)) : 0;
    
    // Sending HTTP Header
    Standard_HTTP_Header();
    
    // Sending HTML document
    Standard_HTML_Top("Island Information");
    
    get_room_info(room, name, &flags);
    
    printf("<TABLE Border=1 Align=Center CellSpacing=0>\n"
           "<TR> <TD ColsPan=\"2\" Align=\"Center\" BGColor=\"6699EE\"><Big><B>%s</B></Big><BR></TD> </TR>\n"
           "<TR> <TD ColsPan=\"2\"><B>Room attributes</B></TD> </TR>\n", 
           name
    );
    
    puts("<TR>\n<TD>");
    printf("<IMG Src=\"/images/%sbox.gif\"> Subject<BR>\n",     (flags & R_subject) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Anonymous<BR>\n",   (flags & R_anonym ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Invite only<BR>\n", (flags & R_invite ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Invisible<BR>\n",   (flags & R_invisi ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Protected<BR>\n",   (flags & R_1_only ) ? "check" : "plain");
    puts("</TD>\n<TD>");
    printf("<IMG Src=\"/images/%sbox.gif\"> Male only<BR>\n",   (flags & R_male   ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Female only<BR>\n", (flags & R_female ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Scrambled<BR>\n",   (flags & R_scram  ) ? "check" : "plain");
    printf("<IMG Src=\"/images/%sbox.gif\"> Inversed<BR>\n",    (flags & R_inv    ) ? "check" : "plain");
    
    puts("</TD>\n</TR>");
    
    // We first try to get the room info in the current language. 
    // If that fails, we try to get the room info in the first 
    // available language (by passing "*").
    
    path = get_room_description(room, lang);
    file = fopen(path, "r");
    
    if(file == NULL) {
        path = get_room_description(room, "*");
        file = fopen(path, "r");
    }
    
    if(file != NULL) {
        puts("<TR>\n<TD BGColor=\"#333333\"><FONT Color=\"#FFFFFF\"><PRE>");
        
        while((n = fread(str, sizeof(char), 1024, file)) != 0) {
            str[n] = 0;
            html = bbs2html(str);
            fwrite(html, sizeof(char), strlen(html), stdout);
            delete html;
        }
        
        puts("</PRE></FONT></TD>\n</TR>");
    }
    
    puts("</TABLE>");
    
    Standard_HTML_Bottom(program, version);
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
    fprintf(stderr, "%s: terminate()\n", program);
    exit(1);
}

