/* ***************************************************************** *
 * Atlantis BBS - BBS to HTML                                        *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.1 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Mon 18 Jan 1999 *
 * Modification date . . . . . . . . . . . . . . . . Fri 26 Mar 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_colors.h"
#include "CGI_HTML.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    1024


/* ***************************************************************** *
 * prototypes                                                        *
 * ***************************************************************** */
void    terminate   (void);
void    translate   (char * title, FILE * in, FILE * out);

 
/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char     program[]           = "BBS to HTML";
static char     version[]           = "1.0.1.3";


/* ***************************************************************** *
 * translate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void translate(char * title, FILE * in, FILE * out) {
    char    str[BUF_SIZE+1];
    char *  html;
    
    Standard_HTML_Top(title);
    
    puts("<TABLE Border=1 BGColor=\"#000000\">\n<TR>\n<TD>\n<PRE>");
    
    while(fgets(str, BUF_SIZE, in)) {
        html = bbs2html(str);
        fprintf(out, html);
        delete html;
    }
    
    puts("</PRE>\n</TD>\n</TR>\n</TABLE>");
    
    Standard_HTML_Bottom("", "");
}


/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(int argc, char * argv[]) {
    FILE *  in = NULL;
    FILE *  out = NULL;
    int     i = 0;
    char    outname[FILENAME_MAX+1];
    char *  title = NULL;
    char *  s = NULL;
    char    help[] = 
    "  bbs2html [options] file\n"\
    "\n"\
    "--help                  Prints a small help and exits.\n"\
    "--title=\"The title\"     Sets the title of the file.\n"\
    "--version               Prints the version number and exits.\n"\
    "\n";
    
    if(argc == 1) {
        fprintf(stderr, "%s", help);
    }
    
    for(i=1; i<argc; i++) {
        if(strstr(argv[i], "--help") != NULL) {
            fprintf(stderr, "%s", help);
            return 0;
        }
        
        if(strstr(argv[i], "--version") != NULL) {
            fprintf(stderr, "%s, version %s\n", program, version);
            return 0;
        }
        
        if(strstr(argv[i], "--title") != NULL) {
            i++;
            title = argv[i];
            /*
            title = strchr(argv[i], '"');
            title++;
            s = strchr(title, '"');
            *s = 0;
            */
            continue;
        }
        
        // argv[i] is the file to be translated. 
        
        strcpy(outname, argv[i]);
        s = strrchr(outname, '.');
        strcpy(s+1, "html");
        
        in = fopen(argv[i], "r");
        if(in == NULL) {
            fprintf(stderr, 
                "bbs2html: can't open '%s'; error %d: %s\n", 
                argv[i], errno, strerror(errno)
            );
        }
        
        out = fopen(outname, "w");
        if(out == NULL) {
            fprintf(stderr, 
                "bbs2html: can't open '%s'; error %d: %s\n", 
                outname, errno, strerror(errno)
            );
        }
        
        fprintf(stderr, "converting %s to %s..\n", argv[i], outname);
        
        translate(title, in, out);
    }
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
    fprintf(stderr, "%s: terminate()\n", program);
    exit(errno);
}

