/* ***************************************************************** *
 * Atlantis BBS - log_error                                          *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.1 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Sun 30 Aug 1998 *
 * Modification date . . . . . . . . . . . . . . . . Fri 28 May 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "paths.h"


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char     program[]           = "log_error";
static char     version[]           = "1.0.1";

FILE *          log_file            = NULL;
static char     filename[]          = LOG_DIR "CGIs";
static char     alt_filename[]      = "/tmp/CGIs";

static int      dont_try            = 0;



/* ***************************************************************** *
 * open_log_file()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This function opens the log file.                               *
 *                                                                   *
 * ***************************************************************** */
void open_log_file(void) {
    
    // We open the log file.
    log_file = fopen(filename, "a");
    
    if(log_file == NULL) {
        // If if unsuccessful, we try to open another
        // file, in the /tmp directory. 
        fprintf(stderr, "%s: can't open '%s', error %d, %s\n", 
                program, filename, 
                errno, strerror(errno)
        );
        
        log_file = fopen(alt_filename, "a");
        
        // If even this attempt failed, we quit. 
        if(log_file == NULL) {
            fprintf(stderr, "%s: can't open '%s', error %d, %s\n", 
                    program, filename, 
                    errno, strerror(errno)
            );
            return;
        }
    }
}


/* ***************************************************************** *
 * log_error()                                                       *
 *                                                                   *
 * ***************************************************************** */
int log_error(char * format, ...) {
    va_list list;
    int     n = 0;
    
    if(dont_try) {
        return 0;
    }
    
    // Is the log file open ?
    if(log_file == NULL) {
        // No, so we open it. 
        log_file = fopen(filename, "a");
        
        if(log_file == NULL) {
            // The file wasn't opened ? We try an alternate file. 
            fprintf(stderr, 
                    "%s: warning: can't open '%s' (error %d); trying alternate file '%s'\n", 
                    program, filename, errno, alt_filename
            );
            log_file = fopen(alt_filename, "a");
            
            if(log_file == NULL) {
                // Even this one can't be opened ??
                // So we quit and will not try again.  
                fprintf(stderr, 
                        "%s: warning: can't open '%s' (error %d); error log desactivated\n", 
                        program, alt_filename, errno 
                );
                dont_try = 1;
                return 0;
            }
        }
        
        setbuf(log_file, NULL);  // Unbuffered
    }
    
    va_start(list, format);
    
    n = vfprintf(log_file, format, list);
    
    va_end(list);
    
    return n;
}

